@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---         Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  QRW Software
+--   Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2011-04-30 19:44:48 -0600 (Sat, 30 Apr 2011) $
+--   $Revision: 4893 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -55,6 +55,7 @@
 #define REFERENCE_H
 
 #include "types.h"
+#include "string.h"
 
 
 #define RESERVED_PUBLIC_BLOCKS    0x10000000    /* IDs 0x00000000 to 0x0fffffff reserved */
@@ -124,7 +125,9 @@ static inline void void_reference(ObjRef* ref)
 /* NOTE: these only work when references are 4 bytes, need to fix when size is changed */
 static inline uint32 nwos_ref_to_word(ObjRef* ref) 
 {
-  return ((uint32)ref->id[0] << 24) | ((uint32)ref->id[1] << 16) | ((uint32)ref->id[2] << 8) | (uint32)ref->id[3];
+    uint8 id[sizeof(ObjRef)];
+    memcpy(id, ref->id, sizeof(id));
+    return ((uint32)id[0] << 24) | ((uint32)id[1] << 16) | ((uint32)id[2] << 8) | (uint32)id[3];
 }
 
 static inline void nwos_word_to_ref(uint32 word, ObjRef* ref) 
