@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2011-12-06 10:47:56 -0700 (Tue, 06 Dec 2011) $
+--   $Revision: 4988 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -87,6 +87,13 @@ extern void nwos_convert_time_stamp_to_timeval(TimeStamp time_stamp, struct time
 
 extern const char* nwos_time_stamp_to_string(TimeStamp time_stamp);
 
+extern bool nwos_time_stamp_match(TimeStamp a, TimeStamp b);    /* if microseconds is FFFFF on either, the microseconds are ignored */
+
+static inline void nwos_time_stamp_copy(TimeStamp dst, TimeStamp src)
+{
+    dst[0] = src[0]; dst[1] = src[1]; dst[2] = src[2]; dst[3] = src[3]; dst[4] = src[4]; dst[5] = src[5]; dst[6] = src[6]; dst[7] = src[7];
+}
+
 
 static inline bool nwos_time_stamp_is_zero(TimeStamp stamp)
 {
@@ -227,6 +234,13 @@ static inline void nwos_insert_microsecond_into_time_stamp(uint usec, TimeStamp
     stamp[7] = usec;
 }
 
+static inline void nwos_set_time_stamp_ignore_microseconds(TimeStamp stamp)
+{
+    stamp[5] = (stamp[5] | 0x0F);
+    stamp[6] = 0xFF;
+    stamp[7] = 0xFF;
+}
+
 
 
 #endif
