@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---               Copyright (C) 2006, 2007, 2008, 2009  QRW Software
+--            Copyright (C) 2006, 2007, 2008, 2009, 2011  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2010-01-21 05:54:56 -0700 (Thu, 21 Jan 2010) $
---   $Revision: 4470 $
+--   $Date: 2011-11-05 17:07:30 -0600 (Sat, 05 Nov 2011) $
+--   $Revision: 4926 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -75,9 +75,21 @@ void print_md5(ObjRef* file_ref)
     int j;
 
     assert(nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj)));  /* read the file object */
-    assert(nwos_read_object_from_disk(&file_obj.md5sum, &md5_object, sizeof(md5_object)));
 
-    for (j = 0; j < MD5_DIGEST_SIZE; j++) printf("%02x", md5_object.md5sum[j]);
+    if (is_void_reference(&file_obj.md5sum))
+    {
+             /*          1         2         3   */
+             /* 12345678901234567890123456789012 */
+
+	printf("**** No MD5 checksum stored ****");
+    }
+    else
+    {
+	assert(nwos_read_object_from_disk(&file_obj.md5sum, &md5_object, sizeof(md5_object)));
+
+	for (j = 0; j < MD5_DIGEST_SIZE; j++) printf("%02x", md5_object.md5sum[j]);
+    }
+
     printf("  ");
 }
 
