@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---                  Copyright (C) 2007, 2008, 2009  QRW Software
+--               Copyright (C) 2007, 2008, 2009, 2010  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2010-01-21 05:54:56 -0700 (Thu, 21 Jan 2010) $
---   $Revision: 4470 $
+--   $Date: 2010-10-08 22:32:02 -0600 (Fri, 08 Oct 2010) $
+--   $Revision: 4786 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -251,8 +251,7 @@ NameChange* read_list_file(char* list_file_name)
 static char* file_is_used_in_disc_list(ObjRef* disc_list_class_ref, C_struct_Path_And_File_Association* assoc_obj)
 {
     ObjRef object_class;
-    uint8 kludge[MAX_SIZE_DISC_LIST];
-    C_struct_Disc_List* ptr_list_obj = (C_struct_Disc_List*)kludge;
+    C_struct_Disc_List* ptr_list_obj;
     ReferenceList* ref_list;
     int num_refs;
     int i;
@@ -260,6 +259,8 @@ static char* file_is_used_in_disc_list(ObjRef* disc_list_class_ref, C_struct_Pat
     char* result = NULL;
 
 
+    ptr_list_obj = malloc(MAX_SIZE_DISC_LIST);
+
     ref_list = nwos_malloc_reference_list(&assoc_obj->header.object.references);
 
     num_refs = ref_list->common_header.num_refs;
@@ -272,7 +273,7 @@ static char* file_is_used_in_disc_list(ObjRef* disc_list_class_ref, C_struct_Pat
 
 	if (is_same_object(&object_class, disc_list_class_ref))
 	{
-	    assert(nwos_read_variable_sized_object_from_disk(&ref_list->references[i], kludge, sizeof(kludge), &nwos_get_disc_list_object_size));
+	    assert(nwos_read_variable_sized_object_from_disk(&ref_list->references[i], ptr_list_obj, MAX_SIZE_DISC_LIST, &nwos_get_disc_list_object_size));
 
 	    memcpy(id, ptr_list_obj->id, sizeof(id));
 	    id[sizeof(ptr_list_obj->id)] = '\0';
@@ -284,6 +285,9 @@ static char* file_is_used_in_disc_list(ObjRef* disc_list_class_ref, C_struct_Pat
     nwos_free_reference_list(ref_list);
     ref_list = NULL;
 
+    nwos_free(ptr_list_obj);
+    ptr_list_obj = NULL;
+
     return result;
 }
 
