@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---               Copyright (C) 2006, 2007, 2008, 2009  QRW Software
+--            Copyright (C) 2006, 2007, 2008, 2009, 2010  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2010-11-10 09:33:49 -0700 (Wed, 10 Nov 2010) $
+--   $Revision: 4814 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -138,6 +138,7 @@ void output_class(ObjRef* class_ref)
     uint8 kludge[MAX_CLASS_DEFINITION_OBJ_SIZE];
     C_struct_Class_Definition* class_def_obj_ptr = (C_struct_Class_Definition*)kludge;
     char name[32];
+    char upper_case_name[32];
     int i;
     int revision;
 
@@ -172,6 +173,9 @@ void output_class(ObjRef* class_ref)
 	    name[2] = 'A';
 	}
 
+	for (i = 0; name[i] != '\0'; i++) upper_case_name[i] = toupper(name[i]);
+	upper_case_name[i] = '\0';
+
 	printf("typedef struct {\n");
 
 	printf("    EveryObject header;\n");
@@ -184,6 +188,8 @@ void output_class(ObjRef* class_ref)
 	if (is_void_reference(&class_def_obj_ptr->header.object.next_version))  /* no newer version */
 	{
 	    printf("} C_struct_%s;\n\n", name);
+
+	    printf("#define NWOS_%s_PUBLIC_REF 0x%08x\n\n", upper_case_name, nwos_ref_to_word(class_ref));
 	}
 	else   /* newer version of this class exists */
 	{
@@ -196,6 +202,8 @@ void output_class(ObjRef* class_ref)
 	    }
 
 	    printf("} C_struct_%s_%03d;\n\n", name, revision);
+
+	    printf("#define NWOS_%s_%03d_PUBLIC_REF 0x%08x\n\n", upper_case_name, revision, nwos_ref_to_word(class_ref));
 	}
     }
 }
