@@ -1,7 +1,7 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---                  Copyright (C) 2008, 2009, 2010  QRW Software
---               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
+--               Copyright (C) 2008, 2009, 2010, 2011  QRW Software
+--                    J. Scott Edwards - qrwsoftware@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
 --   it under the terms of the GNU General Public License as published by
@@ -138,13 +138,14 @@ int process_file(char* file_name)
 	for (i = 0; i < sizeof(SHA1); i++)
 	{
 	    upper = convert_char_to_hex_value(*p++);
+	    lower = 0xff;                /* set lower to something invalid */
 
-	    if (upper < 16)
+	    if (upper < 16)              /* if upper is valid try lower */
 	    {
 		lower = convert_char_to_hex_value(*p++);
 	    }
 
-	    if (upper > 15 || lower > 15)
+	    if (lower > 15)              /* if upper was invalid lower will be invalid too */
 	    {
 		break;
 	    }
