@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2011-11-05 11:04:14 -0600 (Sat, 05 Nov 2011) $
---   $Revision: 4923 $
+--   $Date: 2012-04-15 11:29:04 -0600 (Sun, 15 Apr 2012) $
+--   $Revision: 4996 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -81,7 +81,8 @@ extern void nwos_set_private_objects_path(const char* path);
 
 static void print_usage(char *program)
 {
-    fprintf(stderr, "usage: %s [source-archive] destination-archive\n", program);
+    fprintf(stderr, "usage: %s [--revert] [source-archive] destination-archive\n", program);
+    fprintf(stderr, "  --revert\trevert destination archive back to the previous state of the source archive.\n");
 }
 
 
@@ -108,6 +109,17 @@ static void md5_digest_to_string(uint8* digest, char* string, size_t size)
 }
 
 
+static void clear_chunk(int chunk_info_index)
+{
+    nwos_used_private_blocks = nwos_used_private_blocks - (nwos_chunk_info[chunk_info_index].flags_used & CHUNK_INFO_USED_MASK);
+
+    nwos_chunk_info[chunk_info_index].flags_used = 0;
+    memset(nwos_chunk_info[chunk_info_index].md5_digest, 0, MD5_DIGEST_SIZE);
+
+    nwos_write_empty_chunk(nwos_chunk_info[chunk_info_index].index);
+}
+
+
 int main(int argc, char* argv[])
 {
     int obj_file_desc;
@@ -121,13 +133,16 @@ int main(int argc, char* argv[])
     uint64 num_blocks;
     uint64 used_private_blocks;
     uint32 chunk_block_offset;
-    uint32 blocks_on_disk;
+    uint64 blocks_to_copy;
+    float chunk_percent;
+    float block_percent;
     char answer[8];
     char msg[128];
     Disk_Header disk_header;
     uint32 ref;
     uint32 hash;
-    int chunk_num;
+    int src_info_index;
+    int dst_info_index;
     uint32 new_chunks;
     uint32 used_chunks;
     uint32 total_chunks;
@@ -138,39 +153,53 @@ int main(int argc, char* argv[])
     char* destination;
     size_t source_len = 0;
     bool full_copy = false;
+    bool revert = false;
     uint32 num_chunks_block_size_mismatch;
     uint32 num_chunks_md5_mismatch;
     char source_md5_string[MD5_DIGEST_SIZE * 2 + 1];
     char destination_md5_string[MD5_DIGEST_SIZE * 2 + 1];
 
-
-    if (argc != 2 && argc != 3)
+    if (argc > 1 && *argv[1] == '-')
     {
-	print_usage(argv[0]);
-	exit(1);
+	if (strcmp(argv[1], "--revert") == 0)
+	{
+	    if (argc == 2)
+	    {
+		fprintf(stderr, "error: no archive(s) specified\n");
+		print_usage(argv[0]);
+		exit(1);
+	    }
+
+	    revert = true;
+	}
+	else
+	{
+	    fprintf(stderr, "unknown option: %s\n", argv[1]);
+	    print_usage(argv[0]);
+	    exit(1);
+	}
     }
 
-    if (*argv[1] == '-' || (argc == 3 && *argv[2] == '-'))
+    switch (argc - revert)
     {
-	fprintf(stderr, "error: this program doesn't have any options\n");
-	print_usage(argv[0]);
-	exit(1);
-    }
-
-    if (argc == 2)
-      {
+      case 2:
 	source_len = strlen(nwos_get_private_objects_path()) + 1;
 
 	source = nwos_malloc(source_len);
 
 	strlcpy(source, nwos_get_private_objects_path(), source_len);
 
-	destination = argv[1];
-    }
-    else
-    {
-	source = argv[1];
-	destination = argv[2];
+	destination = argv[1 + revert];
+	break;
+
+      case 3:
+	source = argv[1 + revert];
+	destination = argv[2 + revert];
+	break;
+
+      default:
+	print_usage(argv[0]);
+	exit(1);
     }
 
     if (strcmp(source, destination) == 0)
@@ -191,6 +220,14 @@ int main(int argc, char* argv[])
 
     nwos_terminate_backup(false, NULL);   /* don't create a backup file */
 
+    if (revert && !nwos_archive_is_block_device())
+    {
+	fprintf(stderr, "The --revert option can only be used if the destination is a block device.\n");
+	nwos_terminate_storage();
+	exit(1);
+    }
+
+
     /* See if the number of used blocks is consistent with empty disk */
     if (nwos_used_private_blocks == 0 && nwos_used_private_chunks == 0)
     {
@@ -273,9 +310,6 @@ int main(int argc, char* argv[])
 	}
 #endif
 
-    blocks_on_disk = total_chunks * BLOCKS_IN_CHUNK; 
-
-
     printf("header: %c%c%c%c %c%c%c%c\n",
 	   disk_header.magic_number[0], disk_header.magic_number[1], disk_header.magic_number[2], disk_header.magic_number[3], 
 	   disk_header.version_string[0], disk_header.version_string[1], disk_header.version_string[2], disk_header.version_string[3]);
@@ -284,15 +318,23 @@ int main(int argc, char* argv[])
     printf("Destination archive (%s) - last modified: %s, used blocks: %llu\n", destination, nwos_time_stamp_to_string(nwos_last_change), nwos_used_private_blocks);
 
 
-    if (used_private_blocks < nwos_used_private_blocks)
+    if (!revert && used_private_blocks < nwos_used_private_blocks)
     {
-	fprintf(stderr, "ERROR: the source archive (%s) has fewer blocks (%llu) than the destination archive (%s) archive (%llu).\n",
+	fprintf(stderr, "ERROR: the source archive (%s) has fewer blocks (%llu) than the destination (%s) archive (%llu).\n",
 		source, used_private_blocks, destination, nwos_used_private_blocks);
 	fprintf(stderr, "This version of software cannot remove blocks so something is wrong.\n");
 	nwos_terminate_storage();
 	close(obj_file_desc);
 	exit(1);
     }
+    else if (revert && used_private_blocks > nwos_used_private_blocks)
+    {
+	fprintf(stderr, "ERROR: --revert was specified and the source archive (%s) has more\n", source);
+	fprintf(stderr, "  blocks (%llu) than the destination (%s) archive (%llu).\n", used_private_blocks, destination, nwos_used_private_blocks);
+	nwos_terminate_storage();
+	close(obj_file_desc);
+	exit(1);
+    }
 
 
     /* first make sure everything is kosher */
@@ -316,24 +358,56 @@ int main(int argc, char* argv[])
 
     if (i < nwos_used_private_chunks)   /* refs didn't all match */
     {
-	fprintf(stderr, "ERROR: the destination archive (%s) has chunks that don't exist in the source archive (%s) archive.\n", destination, source);
-	fprintf(stderr, "This means either the archives are different or chunks have been added to the destination since it was sync'd.\n");
-	nwos_terminate_storage();
-	close(obj_file_desc);
-	exit(1);
+	if (revert)
+	{
+	    printf("The destination archive (%s) has chunks that don't exist in the source archive (%s) archive.\n", destination, source);
+	}
+	else
+	{
+	    fprintf(stderr, "ERROR: the destination archive (%s) has chunks that don't exist in the source archive (%s) archive.\n", destination, source);
+	    fprintf(stderr, "This means either the archives are different or chunks have been added to the destination since it was sync'd.\n");
+	    nwos_terminate_storage();
+	    close(obj_file_desc);
+	    exit(1);
+	}
     }
 
     num_chunks_block_size_mismatch = 0;
     num_chunks_md5_mismatch = 0;
 
-    j = 0;
     new_chunks = 0;
     empty_chunks = 0;
-    for (i = 0; i < nwos_used_private_chunks; i++)
+
+    if (revert)
     {
-	while (j < used_chunks && chunk_info[j].ref != nwos_chunk_info[i].ref)
+	i = 0;
+
+	for (j = 0; j < used_chunks; j++)
 	{
-	    if ((chunk_info[j].flags_used & CHUNK_INFO_USED_MASK) == 0)
+	    while (i < nwos_used_private_chunks && nwos_chunk_info[i].ref != chunk_info[j].ref)
+	    {
+		if ((nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK) == 0)
+		{
+		    empty_chunks++;
+		}
+		else
+		{
+		    new_chunks++;
+		}
+		i++;
+	    }
+
+	    if (i == nwos_used_private_chunks) break;
+
+	    if ((chunk_info[j].flags_used & CHUNK_INFO_USED_MASK) != (nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK)) num_chunks_block_size_mismatch++;
+	    if (memcmp(chunk_info[j].md5_digest, nwos_chunk_info[i].md5_digest, MD5_DIGEST_SIZE) != 0) num_chunks_md5_mismatch++;
+
+	    i++;
+	}
+
+	while (i < nwos_used_private_chunks)
+	{
+	    if ((nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK) == 0)
 	    {
 		empty_chunks++;
 	    }
@@ -341,38 +415,145 @@ int main(int argc, char* argv[])
 	    {
 		new_chunks++;
 	    }
+
+	    i++;
+	}
+    }
+    else
+    {
+	/* The cases for each chunk_info entry are:               */
+	/*                                                        */
+	/*    Source and destination references are equal:        */
+	/*                                                        */
+	/*        Counts not equal                                */
+	/*            Source < Destination                        */
+	/*                ERROR - Terminate                       */
+	/*            Source > Destination                        */
+	/*                Increment changed chunk                 */
+	/*                Proceed to next entry both              */
+	/*                                                        */
+	/*        Counts are equal                                */
+	/*            Both are zero                               */
+	/*                Increment empty chunk count             */
+	/*                Proceed to next entry both              */
+	/*            MD5s are equal                              */
+	/*                Proceed to next entry both              */
+	/*            MD5s not equal                              */
+	/*                Increment changed chunk                 */
+	/*                Proceed to next entry both              */
+	/*                                                        */
+	/*    Source and destination references not equal:        */
+	/*                                                        */
+	/*        Source ref > Destination ref                    */
+	/*            Destination is empty                        */
+	/*                Proceed to next destination entry       */
+	/*            Destination not empty                       */
+	/*                ERROR - Terminate                       */
+	/*        Source ref < Destinatipn ref                    */
+	/*            Source is empty                             */
+	/*                Increment empty chunk count             */
+	/*                Proceed to next source entry            */
+	/*            Source not empty                            */
+	/*                Increment new chunk count               */
+	/*                Proceed to next source entry            */
+	/*                                                        */
+
+	j = 0;
+
+	for (i = 0; i < nwos_used_private_chunks; i++)
+	{
+	    while (j < used_chunks && chunk_info[j].ref < nwos_chunk_info[i].ref)      /* source has chunks that don't exist in destination */
+	    {
+		if ((chunk_info[j].flags_used & CHUNK_INFO_USED_MASK) == 0)
+		{
+		    empty_chunks++;
+		}
+		else
+		{
+		    new_chunks++;
+		}
+		j++;
+	    }
+
+	    if (j == used_chunks) break;
+
+	    if (chunk_info[j].ref > nwos_chunk_info[i].ref)                           /* destination has chunks that don't exist in source */
+	    {
+		if ((nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK) != 0)           /* something is wrong if it isn't empty */
+		{
+		    fprintf(stderr, "ERROR: Destination chunk %llx doesn't exist in the source.\n", nwos_chunk_info[i].ref);
+
+		    nwos_terminate_storage();
+		    close(obj_file_desc);
+		    exit(1);
+		}
+	    }
+	    else
+	    {
+		if ((chunk_info[j].flags_used & CHUNK_INFO_USED_MASK) < (nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK))    /* source has fewer blocks */
+		{
+		    fprintf(stderr, "ERROR: Chunk %llx, source has fewer blocks (%u) than destination (%u).\n", 
+			    chunk_info[j].ref, chunk_info[j].flags_used & CHUNK_INFO_USED_MASK, nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK);
+
+		    nwos_terminate_storage();
+		    close(obj_file_desc);
+		    exit(1);
+		}
+		else if ((chunk_info[j].flags_used & CHUNK_INFO_USED_MASK) > (nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK))    /* source has more blocks */
+		{
+		    num_chunks_block_size_mismatch++;
+		}
+		else
+		{
+		    if ((chunk_info[j].flags_used & CHUNK_INFO_USED_MASK) == 0)    /* the source has an empty block (the destination is too since the counts are equal) */
+		    {
+			empty_chunks++;
+		    }
+		    else if (memcmp(chunk_info[j].md5_digest, nwos_chunk_info[i].md5_digest, MD5_DIGEST_SIZE) != 0)
+		    {
+			num_chunks_md5_mismatch++;
+		    }
+		}
+
+		j++;
+	    }
+	}
+
+	while (i < nwos_used_private_chunks)
+	{
+	    if ((nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK) != 0)           /* something is wrong if it isn't empty */
+	    {
+		fprintf(stderr, "ERROR: Destination chunk %llx doesn't exist in the source.\n", nwos_chunk_info[i].ref);
+
+		nwos_terminate_storage();
+		close(obj_file_desc);
+		exit(1);
+	    }
+
+	    i++;
+	}
+
+	while (j < used_chunks)
+	{
+	    if ((chunk_info[j].flags_used & CHUNK_INFO_USED_MASK) == 0)    /* the source has an empty block (the destination is too since the counts are equal) */
+	    {
+		empty_chunks++;
+	    }
+	    else
+	    {
+		new_chunks++;
+	    }
+
 	    j++;
 	}
-
-	if (j == used_chunks) break;
-
-	if ((chunk_info[j].flags_used & CHUNK_INFO_USED_MASK) != (nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK)) num_chunks_block_size_mismatch++;
-	if (memcmp(chunk_info[j].md5_digest, nwos_chunk_info[i].md5_digest, MD5_DIGEST_SIZE) != 0) num_chunks_md5_mismatch++;
-
-	j++;
-    }
-
-    while (j < used_chunks)
-    {
-	if ((chunk_info[j].flags_used & CHUNK_INFO_USED_MASK) == 0)
-	{
-	    empty_chunks++;
-	}
-	else
-	{
-	    new_chunks++;
-	}
-
-	j++;
     }
 
     printf("Number of new chunks: %u\n", new_chunks);
     printf("Number of empty chunks: %u\n", empty_chunks);
-    printf("Number of chunks that changed: %u\n", num_chunks_md5_mismatch);
+    printf("Number of chunks that changed: %u\n", num_chunks_block_size_mismatch + num_chunks_md5_mismatch);
 
-    assert(num_chunks_md5_mismatch >= num_chunks_block_size_mismatch);
 
-    if (new_chunks == 0 && num_chunks_md5_mismatch == 0)
+    if (new_chunks == 0 && num_chunks_block_size_mismatch == 0 && num_chunks_md5_mismatch == 0)
     {
 	printf("\nArchives are the same, sync not needed!\n\n");
 	nwos_terminate_storage();
@@ -413,36 +594,73 @@ int main(int argc, char* argv[])
 	exit(1);
     }
 
-
-    if (nwos_archive_is_block_device())
+    if (revert)     /* go through and clear any chunks in the destination that have more blocks or don't exist in the source */
     {
-	printf("   total blocks on disk: %u  used blocks: %llu\n",
-	       nwos_total_private_chunks * USABLE_BLOCKS_PER_CHUNK, used_private_blocks);
+	i = 0;
 
-	printf("   total chunks on disk: %u  used chunks: %u\n",
-	       nwos_total_private_chunks, used_chunks);
-
-	if (used_chunks > nwos_total_private_chunks)
+	for (j = 0; j < used_chunks; j++)
 	{
-	    fprintf(stderr, "ERROR: used chunks: %u greater than disk space: %u\n",
-		    used_chunks, nwos_total_private_chunks);
-	    exit(1);
+	    while (i < nwos_used_private_chunks && nwos_chunk_info[i].ref != chunk_info[j].ref)
+	    {
+		if ((nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK) != 0)
+		{
+		    clear_chunk(i);
+		}
+		i++;
+	    }
+
+	    if (i == nwos_used_private_chunks) break;
+
+	    if ((chunk_info[j].flags_used & CHUNK_INFO_USED_MASK) < (nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK))
+	    {
+		clear_chunk(i);
+	    }
+
+	    i++;
+	}
+
+	while (i < nwos_used_private_chunks)
+	{
+	    if ((nwos_chunk_info[i].flags_used & CHUNK_INFO_USED_MASK) != 0)
+	    {
+		clear_chunk(i);
+	    }
+
+	    i++;
 	}
     }
-    else if (nwos_total_private_chunks < used_chunks)
+    else
     {
-	printf("Expanding file...\n");
-
-	nwos_start_progress_bar();
-
-	while (nwos_total_private_chunks < used_chunks)
+	if (nwos_archive_is_block_device())
 	{
-	    nwos_update_progress_bar((float)nwos_total_private_chunks / (float)used_chunks);
+	    printf("   total blocks on disk: %u  used blocks: %llu\n",
+		   nwos_total_private_chunks * USABLE_BLOCKS_PER_CHUNK, used_private_blocks);
 
-	    nwos_write_empty_chunk(nwos_total_private_chunks);
-	} 
+	    printf("   total chunks on disk: %u  used chunks: %u\n",
+		   nwos_total_private_chunks, used_chunks);
 
-	nwos_finish_progress_bar();
+	    if (used_chunks > nwos_total_private_chunks)
+	    {
+		fprintf(stderr, "ERROR: used chunks: %u greater than disk space: %u\n",
+		    used_chunks, nwos_total_private_chunks);
+		exit(1);
+	    }
+	}
+	else if (nwos_total_private_chunks < used_chunks)
+	{
+	    printf("Expanding file...\n");
+
+	    nwos_start_progress_bar();
+
+	    while (nwos_total_private_chunks < used_chunks)
+	    {
+		nwos_update_progress_bar((float)nwos_total_private_chunks / (float)used_chunks);
+
+		nwos_write_empty_chunk(nwos_total_private_chunks);
+	    } 
+
+	    nwos_finish_progress_bar();
+	}
     }
 
     if (full_copy)
@@ -462,14 +680,34 @@ int main(int argc, char* argv[])
 	perror("setting SIGINT signal to catch control-C");
     }
 
+    blocks_to_copy = used_private_blocks - nwos_used_private_blocks;
+
     nwos_start_progress_bar();
 
-    // for now we can skip over public blocks because they should always stay the same
-    for (chunk_num = 0; chunk_num < used_chunks && !control_c_received; chunk_num++)
+    for (src_info_index = 0; src_info_index < used_chunks && !control_c_received; src_info_index++)
     {
-	nwos_update_progress_bar((float)chunk_num / (float)used_chunks);
+	if ((chunk_info[src_info_index].flags_used & CHUNK_INFO_USED_MASK) == 0) continue;       /* if the source chunk is empty, skip it */
 
-	chunk = chunk_block_offset + chunk_info[chunk_num].index * BLOCKS_IN_CHUNK;
+	chunk_percent = (float)src_info_index / (float)used_chunks;
+
+	if (revert)
+	{
+	    nwos_update_progress_bar(chunk_percent);
+	}
+	else
+	{
+	    if (blocks_to_copy == 0)
+	    {
+		nwos_update_progress_bar(chunk_percent);     /* blocks to copy is zero, use chunks for 100% of progress */
+	    }
+	    else
+	    {
+		block_percent = (float)(blocks_to_copy - (used_private_blocks - nwos_used_private_blocks)) / (float)blocks_to_copy;
+		nwos_update_progress_bar(block_percent * 0.5 + chunk_percent * 0.5);
+	    }
+	}
+
+	chunk = chunk_block_offset + chunk_info[src_info_index].index * BLOCKS_IN_CHUNK;
 
 	if (lseek(obj_file_desc, chunk << 8, SEEK_SET) < 0)
 	{
@@ -487,7 +725,12 @@ int main(int argc, char* argv[])
 	    exit(1);
 	}
 
-	if (full_copy || chunk_num >= nwos_used_private_chunks || chunk_info[chunk_num].ref != nwos_chunk_info[chunk_num].ref || memcmp(chunk_info[chunk_num].md5_digest, nwos_chunk_info[chunk_num].md5_digest, MD5_DIGEST_SIZE) != 0)
+	for (dst_info_index = 0; dst_info_index < nwos_used_private_chunks; dst_info_index++)    /* the destination chink_info table changes dynamically, find correct index each time */
+	{
+	    if (nwos_chunk_info[dst_info_index].ref >= chunk_info[src_info_index].ref) break;     /* if exact match or destination is past the source stop here */
+	}
+
+	if (full_copy || dst_info_index >= nwos_used_private_chunks || chunk_info[src_info_index].ref != nwos_chunk_info[dst_info_index].ref || memcmp(chunk_info[src_info_index].md5_digest, nwos_chunk_info[dst_info_index].md5_digest, MD5_DIGEST_SIZE) != 0)
 	{
 	    for (i = 0; i < USABLE_BLOCKS_PER_CHUNK; i++)
 	    {
