@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---                  Copyright (C) 2007, 2008, 2009  QRW Software
+--            Copyright (C) 2007, 2008, 2009, 2010, 2011  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2010-01-20 08:10:57 -0700 (Wed, 20 Jan 2010) $
---   $Revision: 4465 $
+--   $Date: 2011-04-30 19:44:48 -0600 (Sat, 30 Apr 2011) $
+--   $Revision: 4893 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -128,6 +128,7 @@ int main(int argc, char* argv[])
     bool ignore_times;
     bool mismatch = false;
     const char* error_msg;
+    Disk_Header* disk_header;
 
 
     if (argc == 4)
@@ -367,7 +368,8 @@ int main(int argc, char* argv[])
 
     if (header2_version >= 30)
     {
-	memcpy(&(((Disk_Header*)buf2)->type_code), TYPE_CODE_COMPRESSED, sizeof((((Disk_Header*)buf2)->type_code)));
+	disk_header = (Disk_Header*)buf2;
+	memcpy(&disk_header->type_code, TYPE_CODE_COMPRESSED, sizeof(disk_header->type_code));
     }
 
     write3 = fwrite(buf2, 1, sizeof(buf2), fp3);
