@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---            Copyright (C) 2005, 2006, 2007, 2008, 2009  QRW Software
+--         Copyright (C) 2005, 2006, 2007, 2008, 2009, 2012  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -17,6 +17,16 @@
 --   along with this program, in the file LICENSE.  If not, see 
 --   <http://www.gnu.org/licenses/>.
 --
+--  In addition, as a special exception, QRW Software Company gives permission
+--  to link the code of this program with any version of the OpenSSL library
+--  which is distributed under a license identical to that listed in the
+--  included LICENSE.OpenSSL file, and distribute linked combinations including
+--  the two.  You must obey the GNU General Public License in all respects
+--  for all of the code used other than OpenSSL.  If you modify this file,
+--  you may extend this exception to your version of the file, but you are not
+--  obligated to do so.  If you do not wish to do so, delete this exception
+--  statement from your version.
+--
 --   For the latest information, source code (SVN), releases, and bug tracking
 --   go to:
 --      http://savannah.nongnu.org/projects/objectify
@@ -39,8 +49,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2012-04-17 22:39:49 -0600 (Tue, 17 Apr 2012) $
+--   $Revision: 5002 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
