@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2010-11-12 11:36:30 -0700 (Fri, 12 Nov 2010) $
---   $Revision: 4838 $
+--   $Date: 2011-11-06 18:56:08 -0700 (Sun, 06 Nov 2011) $
+--   $Revision: 4934 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -398,6 +398,24 @@ void get_assoc_sort_path(ObjRef* assoc_ref, char* name, size_t name_size)
 	insert_character(name, '1', 8, name_size);
     }
 
+    if (has_prefix(name, "Fedora-"))
+    {
+	name[0] = 'f';
+	insert_character(name, '0', 7, name_size);
+    }
+
+    if (has_prefix(name, "FC-"))
+    {
+	name[0] = 'f';
+	name[1] = 'e';
+
+	insert_character(name, 'd', 2, name_size);
+	insert_character(name, 'o', 3, name_size);
+	insert_character(name, 'r', 4, name_size);
+	insert_character(name, 'a', 5, name_size);
+	insert_character(name, '0', 7, name_size);
+    }
+
     if (has_prefix(name, "FC"))
     {
 	name[0] = 'f';
@@ -411,6 +429,18 @@ void get_assoc_sort_path(ObjRef* assoc_ref, char* name, size_t name_size)
 	insert_character(name, '0', 7, name_size);
     }
 
+    if (has_prefix(name, "F-"))
+    {
+	name[0] = 'f';
+
+	insert_character(name, 'e', 1, name_size);
+	insert_character(name, 'd', 2, name_size);
+	insert_character(name, 'o', 3, name_size);
+	insert_character(name, 'r', 4, name_size);
+	insert_character(name, 'a', 5, name_size);
+	insert_character(name, '0', 7, name_size);
+    }
+
     /* Change KNOPPIX to knoppix */
 
     if (has_prefix(name, "KNOPPIX"))
