@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2010-09-05 12:13:19 -0600 (Sun, 05 Sep 2010) $
---   $Revision: 4672 $
+--   $Date: 2011-12-03 11:36:21 -0700 (Sat, 03 Dec 2011) $
+--   $Revision: 4979 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -60,6 +60,7 @@
 
 #include "backup.h"
 #include "disk_io.h"
+#include "error.h"
 #include "log.h"
 #include "time_stamp.h"
 
@@ -136,7 +137,7 @@ bool nwos_initialize_backup(const char* backup_location, Disk_Header* disk_heade
 	if (backup_file_desc < 0)
 	{
 	    fprintf(stderr, "Error creating backup file ");
-	    perror(nwos_backup_path);
+	    nwos_perror(nwos_backup_path);
 	    result = false;
 	}
 	else
@@ -153,7 +154,7 @@ bool nwos_initialize_backup(const char* backup_location, Disk_Header* disk_heade
 
 	    if (write(backup_file_desc, buffer, sizeof(buffer)) != sizeof(buffer))
 	    {
-		perror(nwos_backup_path);
+		nwos_perror(nwos_backup_path);
 		close(backup_file_desc);
 		backup_file_desc = -1;
 		result = false;
@@ -163,7 +164,7 @@ bool nwos_initialize_backup(const char* backup_location, Disk_Header* disk_heade
 
 	    if (result && write(backup_file_desc, buffer, sizeof(buffer)) != sizeof(buffer))
 	    {
-		perror(nwos_backup_path);
+		nwos_perror(nwos_backup_path);
 		close(backup_file_desc);
 		backup_file_desc = -1;
 		result = false;
@@ -184,7 +185,7 @@ void nwos_backup_write_block(uint8 block[FILE_BLOCK_SIZE])
 	snprintf(log_msg, sizeof(log_msg), "write_block write ref:%02x%02x%02x%02x to %s",
 		 block[4], block[5], block[6], block[7], nwos_backup_path);
 
-	perror(log_msg);
+	nwos_perror(log_msg);
 
 	close(backup_file_desc);
 
@@ -542,7 +543,7 @@ void sort_file()
 
     if (errno != 0)   /* exited loop because of read error */
     {
-	perror(nwos_backup_path);
+	nwos_perror(nwos_backup_path);
     }
     else  /* normal end of file exit */
     {
@@ -550,7 +551,7 @@ void sort_file()
 
 	if (lseek(backup_file_desc, (off_t)current_chunk * FILE_BLOCK_SIZE, SEEK_SET) < 0)
 	{
-	    perror(nwos_backup_path);
+	    nwos_perror(nwos_backup_path);
 	    lower_top = -1;                /* stop it from writing anything */
 	    upper_bottom = SORT_BLOCKS;
 	}
@@ -561,7 +562,7 @@ void sort_file()
 
 	    if (write(backup_file_desc, &buffer[0], write_size) != write_size)
 	    {
-		perror(nwos_backup_path);
+		nwos_perror(nwos_backup_path);
 		upper_bottom = SORT_BLOCKS;
 	    }
 	}
@@ -572,7 +573,7 @@ void sort_file()
 
 	    if (write(backup_file_desc, &buffer[upper_bottom * FILE_BLOCK_SIZE], write_size) != write_size)
 	    {
-		perror(nwos_backup_path);
+		nwos_perror(nwos_backup_path);
 	    }
 	}
 
@@ -590,7 +591,7 @@ void sort_file()
 
 	    if (truncate(nwos_backup_path, (off_t)(offset - num_duplicates) * FILE_BLOCK_SIZE) != 0)
 	    {
-		perror(nwos_backup_path);
+		nwos_perror(nwos_backup_path);
 	    }
 	}
     }
@@ -601,7 +602,7 @@ void sort_file()
 
     if (lseek(backup_file_desc, (off_t)2 * FILE_BLOCK_SIZE, SEEK_SET) < 0)
     {
-	perror(nwos_backup_path);
+	nwos_perror(nwos_backup_path);
     }
     else
     {
@@ -622,7 +623,7 @@ void sort_file()
 
 	if (errno != 0)
 	{
-	    perror(nwos_backup_path);
+	    nwos_perror(nwos_backup_path);
 	}
     }
 }
@@ -657,11 +658,11 @@ void nwos_terminate_backup(bool modified, Disk_Header* disk_header)
 
 	    if (lseek(backup_file_desc, (off_t)FILE_BLOCK_SIZE, SEEK_SET) < 0)
 	    {
-		perror(nwos_backup_path);
+		nwos_perror(nwos_backup_path);
 	    }
 	    else if (write(backup_file_desc, &copy_header, sizeof(copy_header)) != sizeof(copy_header))
 	    {
-		perror(nwos_backup_path);
+		nwos_perror(nwos_backup_path);
 	    }
 	    else
 	    {
@@ -672,19 +673,19 @@ void nwos_terminate_backup(bool modified, Disk_Header* disk_header)
 
 	    if (close(backup_file_desc) < 0)
 	    {
-		perror(nwos_backup_path);
+		nwos_perror(nwos_backup_path);
 	    }
 	}
 	else  /* not modified, no need for a backup */
 	{
 	    if (close(backup_file_desc) < 0)
 	    {
-		perror(nwos_backup_path);
+		nwos_perror(nwos_backup_path);
 	    }
 
 	    if (unlink(nwos_backup_path) < 0)
 	    {
-		perror(nwos_backup_path);
+		nwos_perror(nwos_backup_path);
 	    }
 	}
 
