@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2010-06-30 07:54:24 -0600 (Wed, 30 Jun 2010) $
---   $Revision: 4661 $
+--   $Date: 2011-12-03 11:36:21 -0700 (Sat, 03 Dec 2011) $
+--   $Revision: 4979 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -60,6 +60,7 @@
 #include "bit_map.h"
 #include "chunk_info.h"
 #include "disk_io.h"
+#include "error.h"
 #include "gen_id.h"
 #include "header.h"
 #include "log.h"
@@ -137,7 +138,7 @@ static uint32 calculate_chunk_md5sum(Bit_Map_Cache_Entry* entry, uint8 md5_diges
 		if (!nwos_read_block_from_offset(Private_Archive, (int64)(chunk_start_block + BIT_MAP_BLOCKS + (i * 8) + j) * FILE_BLOCK_SIZE, block))
 		{
 		    snprintf(log_msg, sizeof(log_msg), "reading block for chunk MD5 sum: %08x", chunk_start_block + BIT_MAP_BLOCKS + (i * 8) + j);
-		    perror(log_msg);
+		    nwos_perror(log_msg);
 		    exit(1);
 		}
 
