@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2010-02-10 06:23:12 -0700 (Wed, 10 Feb 2010) $
---   $Revision: 4560 $
+--   $Date: 2011-12-03 11:36:21 -0700 (Sat, 03 Dec 2011) $
+--   $Revision: 4979 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -63,6 +63,7 @@
 #include "chunk_info.h"        /* define BLOCKS_IN_CHUNK - THIS needs to be fixed!! */
 #include "config.h"
 #include "disk_io.h"
+#include "error.h"
 #include "header.h"
 #include "log.h"
 #include "mem_alloc.h"
@@ -228,7 +229,7 @@ void index_compressed_file()
     {
 	if (ferror(index_fp))
 	{
-	    perror(index_path);
+	    nwos_perror(index_path);
 	    exit(1);
 	}
 	fclose(index_fp);
@@ -237,7 +238,7 @@ void index_compressed_file()
     {
 	if (errno != 0)
 	{
-	    perror(nwos_private_path);
+	    nwos_perror(nwos_private_path);
 	    exit(1);
 	}
 
@@ -247,7 +248,7 @@ void index_compressed_file()
 	if (index_fp == NULL)
 	{
 	    fprintf(stderr, "Could not create index file: ");
-	    perror(index_path);
+	    nwos_perror(index_path);
 	    fprintf(stderr, "Index will not be saved!\n");
 	}
 	else
@@ -256,19 +257,19 @@ void index_compressed_file()
 
 	    if (fwrite(header, 1, sizeof(header), index_fp) != sizeof(header))
 	    {
-		perror(index_path);
+		nwos_perror(index_path);
 		exit(1);
 	    }
 
 	    if (fwrite(&nwos_storage_index[1], sizeof(ObjRef), i, index_fp) != i)
 	    {
-		perror(index_path);
+		nwos_perror(index_path);
 		exit(1);
 	    }
 
 	    if (fclose(index_fp))
 	    {
-		perror(index_path);
+		nwos_perror(index_path);
 		exit(1);
 	    }
 	}
@@ -342,7 +343,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
     if (public_file_desc < 0)
     {
-	perror(nwos_public_path);
+	nwos_perror(nwos_public_path);
 	exit(1);
     }
 
@@ -354,7 +355,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
     if (fcntl(public_file_desc, F_SETLK, &lock) != 0)
     {
-	perror(nwos_public_path);
+	nwos_perror(nwos_public_path);
 	exit(1);
     }
 #endif
@@ -365,7 +366,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
     if (read(public_file_desc, buffer, sizeof(buffer)) != sizeof(buffer))
     {
 	snprintf(log_msg, sizeof(log_msg), "reading disk header from: %s", nwos_public_path);
-	perror(log_msg);
+	nwos_perror(log_msg);
 	exit(1);
     }
 
@@ -402,7 +403,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
 	if (private_file_desc < 0)
 	{
-	    perror(nwos_private_path);
+	    nwos_perror(nwos_private_path);
 	    exit(1);
 	}
 
@@ -413,7 +414,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
 	if (fcntl(private_file_desc, F_SETLK, &lock) != 0)
 	{
-	    perror(nwos_private_path);
+	    nwos_perror(nwos_private_path);
 	    exit(1);
 	}
 
@@ -426,7 +427,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
 	    if (lseek(private_file_desc, 0LL, SEEK_SET) < 0)
 	    {
-		perror("rewinding after indexing");
+		nwos_perror("rewinding after indexing");
 		exit(1);
 	    }
 	}
@@ -434,7 +435,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 	if (read(private_file_desc, buffer, sizeof(buffer)) != sizeof(buffer))
 	{
 	    snprintf(log_msg, sizeof(log_msg), "reading disk header from: %s", nwos_private_path);
-	    perror(log_msg);
+	    nwos_perror(log_msg);
 	    exit(1);
 	}
 
@@ -508,7 +509,7 @@ bool nwos_terminate_disk_io()
 	if (lseek(private_file_desc, (off_t)0, SEEK_SET) < 0)
 #endif
 	{
-	    perror(nwos_private_path);
+	    nwos_perror(nwos_private_path);
 	    exit(1);
 	}
 
@@ -518,7 +519,7 @@ bool nwos_terminate_disk_io()
 	if (write(private_file_desc, buffer, sizeof(buffer)) != sizeof(buffer))
 #endif
 	{
-	    perror(nwos_private_path);
+	    nwos_perror(nwos_private_path);
 	    exit(1);
 	}
 
@@ -533,7 +534,7 @@ bool nwos_terminate_disk_io()
 
     if (private_file_desc > 0 && close(private_file_desc) < 0)
     {
-	perror(nwos_private_path);
+	nwos_perror(nwos_private_path);
     }
 
     private_file_desc = 0;
@@ -542,7 +543,7 @@ bool nwos_terminate_disk_io()
 
     if (public_file_desc > 0 && close(public_file_desc) < 0)
     {
-	perror(nwos_public_path);
+	nwos_perror(nwos_public_path);
     }
 
     public_file_desc = 0;
@@ -562,14 +563,14 @@ void nwos_read_chunk_info(void* chunk_info, size_t chunk_info_size_in_bytes)
 
     if (lseek(private_file_desc, FILE_BLOCK_SIZE, SEEK_SET) < 0)
     {
-	perror("chunk index");
+	nwos_perror("chunk index");
 	exit(1);
     }
 
     if (read(private_file_desc, nwos_chunk_info, chunk_info_size_in_bytes) != chunk_info_size_in_bytes)
     {
 	snprintf(log_msg, sizeof(log_msg), "reading chunk info from: %s", nwos_private_path);
-	perror(log_msg);
+	nwos_perror(log_msg);
 	exit(1);
     }
 }
@@ -585,14 +586,14 @@ void nwos_write_chunk_info(void* chunk_info, size_t chunk_info_size_in_bytes)
 
     if (lseek(private_file_desc, (off_t)FILE_BLOCK_SIZE, SEEK_SET) < 0)
     {
-	perror(nwos_private_path);
+	nwos_perror(nwos_private_path);
 	exit(1);
     }
 
     if (write(private_file_desc, nwos_chunk_info, chunk_info_size_in_bytes) != chunk_info_size_in_bytes)
     {
 	snprintf(log_msg, sizeof(log_msg), "writing chunk info to: %s", nwos_private_path);
-	perror(log_msg);
+	nwos_perror(log_msg);
 	exit(1);
     }
 }
@@ -624,7 +625,7 @@ void nwos_write_empty_chunk(uint32 chunk_index)
 
 	if (chunk == NULL)
 	{
-	    perror("allocate memory for creating new chunk");
+	    nwos_perror("allocate memory for creating new chunk");
 	    exit(1);
 	}
 
@@ -635,13 +636,13 @@ void nwos_write_empty_chunk(uint32 chunk_index)
 
     if (lseek(private_file_desc, (off_t)offset << 8, SEEK_SET) < 0)
     {
-	perror(nwos_private_path);
+	nwos_perror(nwos_private_path);
 	exit(1);
     }
 
     if (write(private_file_desc, chunk, CHUNK_SIZE) != CHUNK_SIZE)
     {
-	perror(nwos_private_path);
+	nwos_perror(nwos_private_path);
 	exit(1);
     }
 
@@ -666,14 +667,14 @@ void nwos_write_bit_map(int chunk_index, void* map)
     if (lseek(private_file_desc, (off_t)chunk * FILE_BLOCK_SIZE, SEEK_SET) < 0)
     {
 	snprintf(log_msg, sizeof(log_msg), "write_bit_map lseek chunk:%08x", chunk);
-	perror(log_msg);
+	nwos_perror(log_msg);
 	exit(1);
     }
 
     if (write(private_file_desc, map, BIT_MAP_BYTES) != BIT_MAP_BYTES)
     {
 	snprintf(log_msg, sizeof(log_msg), "write_bit_map write chunk:%08x", chunk);
-	perror(log_msg);
+	nwos_perror(log_msg);
 	exit(1);
     }
 
@@ -693,14 +694,14 @@ void nwos_read_bit_map(int chunk_index, void* map)
     if (lseek(private_file_desc, (off_t)chunk * FILE_BLOCK_SIZE, SEEK_SET) < 0)
     {
 	snprintf(log_msg, sizeof(log_msg), "read_bit_map lseek chunk:%08x", chunk);
-	perror(log_msg);
+	nwos_perror(log_msg);
 	exit(1);
     }
 
     if (read(private_file_desc, map, BIT_MAP_BYTES) != BIT_MAP_BYTES)
     {
 	snprintf(log_msg, sizeof(log_msg), "read_bit_map write chunk:%08x", chunk);
-	perror(log_msg);
+	nwos_perror(log_msg);
 	exit(1);
     }
 }
@@ -759,7 +760,7 @@ bool nwos_read_block_from_offset(Archive_Type archive, int64 offset, uint8 block
 	uint32 offset_lwr = (uint32)offset;
 	snprintf(ref_str, sizeof(ref_str), "nwos_read_block_from_offset lseek offset:%x%08x", offset_upr, offset_lwr);
 
-	perror(ref_str);
+	nwos_perror(ref_str);
 	exit(1);
     }
 
@@ -794,7 +795,7 @@ bool nwos_write_block_to_offset(Archive_Type archive, int64 offset, uint8 block[
 	uint32 offset_lwr = (uint32)offset;
 	snprintf(log_msg, sizeof(log_msg), "nwos_write_block_to_offset lseek offset:%x%08x", offset_upr, offset_lwr);
 
-	perror(log_msg);
+	nwos_perror(log_msg);
 	exit(1);
     }
 
