@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---               Copyright (C) 2006, 2007, 2008, 2009  QRW Software
+--            Copyright (C) 2006, 2007, 2008, 2009, 2010  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -17,8 +17,11 @@
 --   along with this program, in the file LICENSE.  If not, see 
 --   <http://www.gnu.org/licenses/>.
 --
---   For the latest information, source code (SVN), releases, bug and feature
---   request tracking go to:
+--   For the latest information, source code (SVN), releases, and bug tracking
+--   go to:
+--      http://savannah.nongnu.org/projects/objectify
+--
+--   For releases from Alpha_30 and up, bug and feature request tracking go to:
 --      http://sourceforge.net/projects/objectify
 --
 --   For older bug tracking, releases and source code (CVS) prior to the
@@ -36,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-08-13 20:06:13 -0600 (Thu, 13 Aug 2009) $
---   $Revision: 4329 $
+--   $Date: 2010-01-17 14:46:04 -0700 (Sun, 17 Jan 2010) $
+--   $Revision: 4460 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -68,19 +71,11 @@ ObjRef nwos_next_public_ref;
 
 #ifndef PUBLIC_MODE
 
-#ifdef CHANGE_SECURITY_TO_DENSITY
-static Security_Level security_level = Security_None;
-#endif
 static uint32 block_estimate;
 static uint32 last_used_blocks;
 
 static ObjRef last_ref;
 
-#ifdef CHANGE_SECURITY_TO_DENSITY
-static uint32 chunk_density_target;
-static uint32 chunk_skip_forward;
-#endif
-
 #endif
 
 
@@ -148,67 +143,6 @@ void nwos_terminate_gen_id(void)
 
 #ifndef PUBLIC_MODE
 
-#ifdef CHANGE_SECURITY_TO_DENSITY
-static void nwos_set_allocation_parameters(uint32 density, uint32 skip_forward)
-{
-    assert(1 <= density && density <= USABLE_BLOCKS_PER_CHUNK);
-    assert(1 <= skip_forward && skip_forward <= RESERVED_PRIVATE_CHUNKS);
-
-    chunk_density_target = density;
-    chunk_skip_forward = skip_forward;
-}
-
-
-void nwos_restart_id_generation()
-{
-    void_reference(&last_ref);
-}
-
-
-void nwos_set_security_level(Security_Level new_level)
-{
-    bool Security_Not_Set = false;
-
-    assert(Security_None < new_level && new_level <= Security_Extreme);
-
-    security_level = new_level;
-
-    switch (new_level)
-    {
-      case Security_Minimal:
-	nwos_set_allocation_parameters( ((USABLE_BLOCKS_PER_CHUNK * 7) / 8), 1);
-	break;
-
-      case Security_Very_Low:
-	nwos_set_allocation_parameters( (USABLE_BLOCKS_PER_CHUNK / SECURITY_VERY_LOW_N), 4);
-	break;
-
-      case Security_Low:
-	nwos_set_allocation_parameters( (USABLE_BLOCKS_PER_CHUNK / SECURITY_LOW_N), 16);
-	break;
-
-      case Security_Medium:
-	nwos_set_allocation_parameters( (USABLE_BLOCKS_PER_CHUNK / SECURITY_MEDIUM_N), 32);
-	break;
-
-      case Security_High:
-	nwos_set_allocation_parameters( (USABLE_BLOCKS_PER_CHUNK / SECURITY_HIGH_N), 128);
-	break;
-
-      case Security_Extreme:
-	nwos_set_allocation_parameters(1, RESERVED_PRIVATE_CHUNKS);
-	break;
-
-      default:
-	assert(Security_Not_Set);
-	break;
-    }
-
-    nwos_restart_id_generation();   /* clear last_ref so this security level starts fresh */
-}
-#endif  /* CHANGE_SECURITY_TO_DENSITY */
-
-
 /*************************/
 /* Block estimate stuff. */
 /*************************/
@@ -378,10 +312,6 @@ bool nwos_check_blocks_available(uint32 estimated_blocks)
     uint32 needed_blocks;
     uint32 needed_chunks;
     uint32 allocated_blocks;
-#ifdef CHANGE_SECURITY_TO_DENSITY
-    uint32 skip;
-    uint32 max;
-#endif
     uint64 ref;
     uint32 offset;
     char msg[256];
@@ -392,11 +322,6 @@ bool nwos_check_blocks_available(uint32 estimated_blocks)
 
     assert(estimated_blocks > 0);
 
-#ifdef CHANGE_SECURITY_TO_DENSITY
-    assert(1 <= chunk_density_target && chunk_density_target <= USABLE_BLOCKS_PER_CHUNK);
-    assert(1 <= chunk_skip_forward && chunk_skip_forward <= RESERVED_PRIVATE_CHUNKS);
-#endif
-
     if (block_estimate > 0)   /* there was a previous estimate */
     {
 	log_estimate_results();
@@ -479,30 +404,11 @@ bool nwos_check_blocks_available(uint32 estimated_blocks)
 
 	    ref = nwos_chunk_info[i].ref - USABLE_BLOCKS_PER_CHUNK;
 	}
-#ifdef CHANGE_SECURITY_TO_DENSITY
-	else if (chunk_skip_forward == 1)
-	{
-	    ref = nwos_chunk_info[nwos_used_private_chunks-1].ref + USABLE_BLOCKS_PER_CHUNK;
-	}
-	else if (chunk_skip_forward == RESERVED_PRIVATE_CHUNKS)
-	{
-	    max = MAXIMUM_32_BIT_PRIVATE_REFERENCE - nwos_chunk_info[nwos_used_private_chunks-1].ref;
-	    skip = 1 + random() % (max / USABLE_BLOCKS_PER_CHUNK);
-	    ref = nwos_chunk_info[nwos_used_private_chunks-1].ref + (USABLE_BLOCKS_PER_CHUNK * skip);
-	}
-	else
-	{
-	    max = (MAXIMUM_32_BIT_PRIVATE_REFERENCE - nwos_chunk_info[nwos_used_private_chunks-1].ref) / USABLE_BLOCKS_PER_CHUNK;
-	    if (max > chunk_skip_forward) max = chunk_skip_forward;
-	    skip = 1 + random() % max;
-	    ref = nwos_chunk_info[nwos_used_private_chunks-1].ref + (USABLE_BLOCKS_PER_CHUNK * skip);
-	}
-#else
 	else
 	{
 	    ref = nwos_chunk_info[nwos_used_private_chunks-1].ref + USABLE_BLOCKS_PER_CHUNK;
 	}
-#endif
+
 	snprintf(msg, sizeof(msg), "Allocating chunk: %llx", ref);
 	nwos_log(msg);
 
@@ -637,247 +543,21 @@ bool nwos_check_blocks_available(uint32 estimated_blocks)
 #ifdef PUBLIC_MODE
 void nwos_generate_new_public_id(ObjRef* ref)
 {
-    int i;
-    int acc;
-    uint8 diff[4];
+    uint8 next_ref_64[8];
+
+    nwos_uint64_to_8_uint8(&nwos_used_public_blocks, next_ref_64);
+
+    nwos_next_public_ref.id[0] = next_ref_64[4];
+    nwos_next_public_ref.id[1] = next_ref_64[5];
+    nwos_next_public_ref.id[2] = next_ref_64[6];
+    nwos_next_public_ref.id[3] = next_ref_64[7];
 
     copy_reference(ref, &nwos_next_public_ref);
 
-    for (i = sizeof(nwos_next_public_ref)-1; i > -1; i--)
-    {
-	nwos_next_public_ref.id[i]++;
-	if (nwos_next_public_ref.id[i] != 0) break;
-    }
-
-    acc = (int)nwos_next_public_ref.id[3] - (int)ref->id[3];
-    diff[3] = acc;
-    acc = (int)nwos_next_public_ref.id[2] - (int)ref->id[2] + (acc >> 8);
-    diff[2] = acc;
-    acc = (int)nwos_next_public_ref.id[1] - (int)ref->id[1] + (acc >> 8);
-    diff[1] = acc;
-    acc = (int)nwos_next_public_ref.id[0] - (int)ref->id[0] + (acc >> 8);
-    diff[0] = acc;
-
-    assert(diff[0] == 0 && diff[1] == 0 && diff[2] == 0 && diff[3] == 1);
-
     nwos_used_public_blocks++;
 }
 #else
 
-
-#ifdef CHANGE_SECURITY_TO_DENSITY
-
-static void generate_new_completely_random_id(ObjRef* ref)
-{
-    int i;
-    int info_index = 0;
-    uint32 word;
-#ifdef LOG_ID_GENERATION
-    char msg[128];
-#endif
-
-    assert(nwos_used_private_chunks > 0);
-
-    if (nwos_used_private_chunks > 1)
-    {
-	/* first try to find one with some empties at random */
-	for (i = 0; i < nwos_used_private_chunks; i++)
-	{
-	    info_index = random() % nwos_used_private_chunks;
-
-	    if (!nwos_chunk_is_full(chunk)) break;
-	}
-
-	if (i == nwos_used_private_chunks)   /* didn't find one, just do a search */
-	{
-	    for (info_index = 0; info_index < nwos_used_private_chunks; info_index++)
-	    {
-		if (!nwos_chunk_is_full(chunk)) break;
-	    }
-	}
-    }
-
-    assert(!nwos_chunk_is_full(chunk));
-
-    word = nwos_find_random_block_in_chunk(info_index);
-
-    nwos_word_to_ref(word, ref);
-
-#ifdef LOG_ID_GENERATION
-    snprintf(msg, sizeof(msg), "generate_new_completely_random_id(%08x)", word);
-    nwos_log(msg);
-#endif
-
-    assert(!nwos_block_used(ref));
-}
-
-
-static void nwos_generate_new_completely_random_id(ObjRef* ref)
-{
-    generate_new_completely_random_id(ref);
-    nwos_set_bit_in_map(nwos_hash_ref(ref));
-}
-
-
-
-/* density is how many blocks to use 1-7 out of 8 blocks. */
-
-static void nwos_generate_new_closely_spaced_id(int density, ObjRef* new_ref)
-{
-    uint32 ref;
-    uint32 block;
-    int    byte_num = 0;
-    int    max_used;
-    uint16 chunk_index;
-    static int info_index = 0;
-#ifdef LOG_ID_GENERATION
-    char   msg[128];
-#endif
-
-    assert(1 <= density && density <= 7);
-    assert(block_estimate > 0);
-
-    max_used = (USABLE_BLOCKS_PER_CHUNK * (8 - density)) / 8;
-
-    /* if the estimated blocks will fit in one chunk, reduce it to what we need */
-    if (block_estimate <= USABLE_BLOCKS_PER_CHUNK - max_used)
-    {
-	max_used = USABLE_BLOCKS_PER_CHUNK - block_estimate;
-    }
-
-    if (is_void_reference(&last_ref))
-    {
-	block = nwos_block_offset_to_chunks + nwos_chunk_info[info_index].index * BLOCKS_IN_CHUNK;
-    }
-    else
-    {
-	block = nwos_hash_ref(&last_ref) + 1;
-    }
-
-
-    /* at this point 'block' should have the block number we are starting the search at */
-
-    while (true)
-    {
-	/* first see if block has gone past the end of this chunk */
-
-	if ((block - nwos_block_offset_to_chunks) % BLOCKS_IN_CHUNK < BIT_MAP_BLOCKS)
-	{
-	    chunk_index = (block - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK;
-
-	    info_index = nwos_chunk_index_to_info_index(chunk_index);
-
-	    while (nwos_blocks_used_in_chunk(info_index) > max_used)
-	    {
-		/* see if the chunk after this one is usable, and if so back into this chunk */
-		if (info_index + 1 == nwos_total_private_chunks)
-		{
-		    /* need to allocate a new chunk */
-		    bool need_to_fix_allocation_code = false;
-		    assert(need_to_fix_allocation_code);
-		}
-		else if (nwos_blocks_used_in_chunk(info_index + 1) <= max_used)
-		{
-		    byte_num = nwos_find_low_density_in_chunk(info_index, density);
-
-		    if (BIT_MAP_BYTES - byte_num > 1) break;
-
-		    byte_num = 0;    /* reset this to what it was */
-		}
-
-		info_index++;
-	    }
-
-	    block = nwos_block_offset_to_chunks + nwos_chunk_info[info_index].index * BLOCKS_IN_CHUNK + BIT_MAP_BLOCKS + (byte_num * 8);
-	}
-
-	/* at this point we have the block to try */
-
-	block = nwos_find_next_empty_block(block, density);
-
-	if ((block - nwos_block_offset_to_chunks) % BLOCKS_IN_CHUNK >= BIT_MAP_BLOCKS)
-	{
-	    break;  /* found an empty in that chunk */
-	}
-    }
-
-    assert(nwos_test_bit_in_map(block) == false);
-
-    nwos_set_bit_in_map(block);
-
-    ref = nwos_chunk_info[info_index].ref + (block - nwos_block_offset_to_chunks - nwos_chunk_info[info_index].index * BLOCKS_IN_CHUNK - BIT_MAP_BLOCKS);
-
-    nwos_word_to_ref(ref, new_ref);
-
-    copy_reference(&last_ref, new_ref);
-
-#ifdef LOG_ID_GENERATION
-    snprintf(msg, sizeof(msg), "nwos_generate_new_closely_spaced_id(%08x)", ref);
-    nwos_log(msg);
-#endif
-}
-
-
-/* generate approximately 1 block out of N */
-
-static void nwos_generate_new_1_in_N_id(uint32 n, ObjRef* new_ref)
-{
-    uint32 ref;
-    uint32 hash;
-#ifdef LOG_ID_GENERATION
-    char   msg[128];
-#endif
-
-
-    if (is_void_reference(&last_ref))
-    {
-	generate_new_completely_random_id(new_ref);
-
-	ref = nwos_ref_to_word(new_ref);
-    }
-    else
-    {
-	ref = nwos_ref_to_word(&last_ref) + (random() % n) + 1;
-    }
-
-    while (true)
-    {
-	hash = nwos_hash_uint32_ref(ref);
-
-	if (hash == 0)  /* continue this somewhere else */
-	{
-	    generate_new_completely_random_id(new_ref);
-
-	    ref = nwos_ref_to_word(new_ref);
-
-	    hash = nwos_hash_uint32_ref(ref);
-	}	    
-	else if (!nwos_test_bit_in_map(hash))   /* available */
-	{
-	    break;
-	}
-
-	ref++;    /* try the next one */
-    }
-
-    assert(hash == nwos_hash_uint32_ref(ref));
-    assert(!nwos_test_bit_in_map(hash));
-
-    nwos_set_bit_in_map(hash);
-
-    nwos_word_to_ref(ref, new_ref);
-
-    copy_reference(&last_ref, new_ref);
-
-#ifdef LOG_ID_GENERATION
-    snprintf(msg, sizeof(msg), "nwos_generate_new_1_in_N_id(%08x)", ref);
-    nwos_log(msg);
-#endif
-}
-
-#endif  /* CHANGE_SECURITY_TO_DENSITY */
-
-
 void nwos_generate_new_private_id(ObjRef* new_ref)
 {
     uint32 ref;
@@ -951,51 +631,3 @@ void nwos_generate_new_id(ObjRef* ref)
 }
 
 
-/* This is the old code from nwos_generate_new_id that needs to be changed to control density not security */
-#ifdef CHANGE_SECURITY_TO_DENSITY
-#if 0
-    bool Cant_Use_Security_None = false;
-#endif
-    bool Security_Not_Set = false;
-
-    switch (security_level)
-    {
-#if 0
-      case Security_None:
-	assert(Cant_Use_Security_None);
-	break;
-#endif
-
-      case Security_Minimal:
-	nwos_generate_new_closely_spaced_id(7, ref);
-	break;
-
-      case Security_Very_Low:
-	nwos_generate_new_1_in_N_id(SECURITY_VERY_LOW_N, ref);
-	break;
-
-      case Security_Low:
-	nwos_generate_new_1_in_N_id(SECURITY_LOW_N, ref);
-	break;
-
-      case Security_Medium:
-	nwos_generate_new_1_in_N_id(SECURITY_MEDIUM_N, ref);
-	break;
-
-      case Security_High:
-	nwos_generate_new_1_in_N_id(SECURITY_HIGH_N, ref);
-	break;
-
-      default:
-	assert(Security_Not_Set);
-	break;
-
-      case Security_Extreme:
-	nwos_generate_new_completely_random_id(ref);
-	break;
-    }
-
-    assert(nwos_reference_type(ref) == Private_Reference);
-#endif
-
-
