@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2011-12-03 11:36:21 -0700 (Sat, 03 Dec 2011) $
+--   $Revision: 4979 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -52,6 +52,7 @@
 #include <assert.h>
 #include <stdio.h>
 
+#include "error.h"
 #include "mem_alloc.h"
 
 
@@ -77,7 +78,7 @@ void* nwos_malloc(size_t size)
     {
 	char msg[64];
 	snprintf(msg, sizeof(msg), "%zd bytes", size);
-	perror(msg);
+	nwos_perror(msg);
 	assert(result);
 	exit(1);
     }
@@ -109,7 +110,7 @@ void* nwos_realloc(void* oldptr, size_t size)
     {
 	char msg[64];
 	snprintf(msg, sizeof(msg), "%zd bytes", size);
-	perror(msg);
+	nwos_perror(msg);
 	assert(result);
 	exit(1);
     }
