@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---      Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010  QRW Software
+--   Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2010-06-30 07:54:24 -0600 (Wed, 30 Jun 2010) $
---   $Revision: 4661 $
+--   $Date: 2011-12-10 09:03:30 -0700 (Sat, 10 Dec 2011) $
+--   $Revision: 4992 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -58,12 +58,17 @@
 #include "bit_map.h"
 #include "chunk_info.h"
 #include "disk_io.h"
+#include "error.h"
 #include "gen_id.h"
 #include "log.h"
 #include "storage.h"
 #include "user_config.h"
 
 
+static const char* save_refs_path;
+static FILE* save_refs_fp;
+
+
 
 /**********************************************/
 /* Function to initialize the storage modules */
@@ -162,6 +167,9 @@ void nwos_flush_storage(void)
 
 void nwos_terminate_storage(void)
 {
+#ifdef NO_BACKUP
+    nwos_terminate_disk_io();
+#else
     bool disk_was_modified;
 
 #ifndef PUBLIC_MODE
@@ -177,17 +185,53 @@ void nwos_terminate_storage(void)
 
     nwos_terminate_gen_id();
 
-#ifndef NO_BACKUP
 #ifdef PUBLIC_MODE
     nwos_terminate_backup(disk_was_modified, &nwos_public_disk_header);
 #else
     nwos_terminate_backup(disk_was_modified, &nwos_private_disk_header);
 #endif
 #endif
+
+    if (save_refs_fp)
+    {
+	if (fclose(save_refs_fp) != 0)
+	{
+	    perror(save_refs_path);
+	}
+
+	save_refs_fp = NULL;
+    }
 }
 
 
 
+/************************************************/
+/* Activate the saving of each block referenced */
+/************************************************/
+
+void nwos_save_refs(const char* path)
+{
+    assert((path != NULL) ^ (save_refs_path != NULL));
+    //    assert(nwos_disk_io_is_read_only());                  /* this function has to be called before initailize */
+
+    if (path == NULL)
+    {
+	assert(save_refs_fp != NULL);
+
+	if (fclose(save_refs_fp) != 0)
+	{
+	    nwos_perror(save_refs_path);
+	    nwos_terminate_storage();
+	    exit(1);
+	}
+
+	save_refs_fp = NULL;
+    }
+
+    save_refs_path = path;
+}
+
+
 /*****************************************/
 /* Function to read a block from storage */
 /*****************************************/
@@ -314,6 +358,42 @@ bool nwos_read_block(ObjRef* ref, uint8 block[FILE_BLOCK_SIZE])
 	else
 	{
 	    assert((block[0] == 0xff) & (block[1] == 0xff) & (block[2] == 0xff) & (block[3] == 0xfe));
+
+	    static ObjRef last_ref;
+
+	    if (save_refs_path != NULL && save_refs_fp == NULL)
+	    {
+		if (!is_void_reference(&last_ref) && ((last_ref.id[0] & 1) != (ref->id[0] & 1) || last_ref.id[1] != ref->id[1] || last_ref.id[2] != ref->id[2] || last_ref.id[3] != ref->id[3]))
+		{
+		    fprintf(stderr, "found root: %08x\n", nwos_ref_to_word(&last_ref));
+		    save_refs_fp = fopen(save_refs_path, "w");
+		    if (save_refs_fp == NULL)
+		    {
+			perror(save_refs_path);
+			exit(1);
+		    }
+		    if (fwrite(&last_ref, 1, 4, save_refs_fp) != 4)
+		    {
+			perror(save_refs_path);
+			exit(1);
+		    }
+		}
+		else
+		{
+		    copy_reference(&last_ref, ref);
+		}
+	    }
+
+	    if (save_refs_fp != NULL)
+	    {
+//		fprintf(stderr, "read_block: %08x\n", nwos_ref_to_word(ref));
+		if (fwrite(ref, 1, 4, save_refs_fp) != 4)
+		{
+		    perror(save_refs_path);
+		    fclose(save_refs_fp);
+		    exit(1);
+		}
+	    }
 	}
 
 	assert((block[4] == ref->id[0]) & (block[5] == ref->id[1]) & (block[6] == ref->id[2]) & (block[7] == ref->id[3]));
@@ -379,7 +459,7 @@ void nwos_write_block(ObjRef* ref, uint8 block[FILE_BLOCK_SIZE])
     {
 	snprintf(log_msg, sizeof(log_msg), "nwos_write_block write ref:%02x%02x%02x%02x", ref->id[0], ref->id[1], ref->id[2], ref->id[3]);
 
-	perror(log_msg);
+	nwos_perror(log_msg);
 	exit(1);
     }
 
