@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2011-12-06 10:47:56 -0700 (Tue, 06 Dec 2011) $
+--   $Revision: 4988 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -52,6 +52,9 @@
 #include <stdio.h>
 #include <time.h>
 #include <stdlib.h>
+#include <string.h>
+
+#include "error.h"
 #include "time_stamp.h"
 
   /*
@@ -95,13 +98,13 @@ void nwos_get_time_stamp(TimeStamp stamp)
   struct timespec ts;
 #ifdef HAVE_CLOCK_GETTIME
   if (clock_gettime(CLOCK_REALTIME, &ts) != 0) {
-    perror("reading real-time clock");
+    nwos_perror("reading real-time clock");
     exit(1);
   }
 #else
   struct timeval tv;
   if (gettimeofday(&tv, NULL) != 0) {
-    perror("reading real-time clock");
+    nwos_perror("reading real-time clock");
     exit(1);
   }
   ts.tv_sec = tv.tv_sec;
@@ -125,7 +128,7 @@ void nwos_convert_timespec_to_time_stamp(struct timespec* ts, TimeStamp stamp)
   usec = ts->tv_nsec / 1000;   /* convert to microseconds */
 
   if (gmtime_r(&ts->tv_sec, &bdt) == NULL) {
-    perror("gmtime_r");
+    nwos_perror("gmtime_r");
     exit(1);
   }
 
@@ -160,6 +163,31 @@ const char* nwos_time_stamp_to_string(TimeStamp ts)
     return result;
 }
 
+/* if microseconds is FFFFF on either, the microseconds are ignored */
+
+bool nwos_time_stamp_match(TimeStamp a, TimeStamp b)
+{
+    TimeStamp copy;
+
+    if (memcmp(a, b, sizeof(TimeStamp)) == 0) return true;      /* if it all matches we are good to go */
+
+    if (((a[5] & 0x0f) == 0x0f) && (a[6] == 0xff) && (a[7] == 0xff))    /* a is set to ignore microseconds */
+    {
+	nwos_time_stamp_copy(copy, b);
+	nwos_set_time_stamp_ignore_microseconds(copy);
+	return (memcmp(a, copy, sizeof(TimeStamp)) == 0);
+    }
+
+    if (((b[5] & 0x0f) == 0x0f) && (b[6] == 0xff) && (b[7] == 0xff))    /* b is set to ignore microseconds */
+    {
+	nwos_time_stamp_copy(copy, a);
+	nwos_set_time_stamp_ignore_microseconds(copy);
+	return (memcmp(b, copy, sizeof(TimeStamp)) == 0);
+    }
+
+    return false;
+}
+
 
 /* Standalone test function - compile with -DTEST_ITER=n */
 
@@ -199,7 +227,7 @@ int main()
 	sec = rand() & 0x7fffffff;  /* always positive */
 
 	if (gmtime_r(&sec, &bdt1) == NULL) {
-	  perror("gmtime_r");
+	  nwos_perror("gmtime_r");
 	  exit(1);
 	}
 
