@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---               Copyright (C) 2006, 2007, 2008, 2009  QRW Software
+--            Copyright (C) 2006, 2007, 2008, 2009, 2010  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2010-10-15 03:40:26 -0600 (Fri, 15 Oct 2010) $
+--   $Revision: 4790 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -82,8 +82,7 @@ size_t nwos_get_disc_list_object_size(void* disc_list_obj)
 bool nwos_find_disc_list(char id[12], ObjRef* ref)
 {
     C_struct_Class_Definition class_def_obj;
-    uint8 kludge[MAX_SIZE_DISC_LIST];
-    C_struct_Disc_List* ptr_disc_obj = (C_struct_Disc_List*)kludge;
+    C_struct_Disc_List* ptr_disc_obj;
     ObjRef disc_class_ref;
     ObjRef object_class;
     ReferenceList* ref_list;
@@ -106,15 +105,15 @@ bool nwos_find_disc_list(char id[12], ObjRef* ref)
 
     /* printf("num_refs: %d\n", num_refs); */
 
+    ptr_disc_obj = nwos_malloc(MAX_SIZE_DISC_LIST);
+
     for (i = 0; i < num_refs; i++)
     {
 	nwos_get_object_class(&ref_list->references[i], &object_class);
 
 	if (is_same_object(&object_class, &disc_class_ref))
 	{
-	    assert(nwos_read_variable_sized_object_from_disk(&ref_list->references[i], kludge, sizeof(kludge), &nwos_get_disc_list_object_size));
-
-	    /* remember ptr_disc_obj points to the kludge buffer */
+	    assert(nwos_read_variable_sized_object_from_disk(&ref_list->references[i], ptr_disc_obj, MAX_SIZE_DISC_LIST, &nwos_get_disc_list_object_size));
 
 	    if (memcmp(ptr_disc_obj->id, id, sizeof(ptr_disc_obj->id)) == 0)   /* found a match */
 	    {
@@ -124,6 +123,9 @@ bool nwos_find_disc_list(char id[12], ObjRef* ref)
 	}
     }
 
+    nwos_free(ptr_disc_obj);
+    ptr_disc_obj = NULL;
+
     nwos_free_reference_list(ref_list);
     ref_list = NULL;
 
@@ -311,8 +313,7 @@ uint32 nwos_read_files_disc_list(const char* root_dir, const char* subdirectory,
 bool nwos_find_matching_disc_list(ObjRef files[MAX_FILES_PER_DISC_LIST], ObjRef* ref)
 {
     C_struct_Class_Definition class_def_obj;
-    uint8 kludge[MAX_SIZE_DISC_LIST];
-    C_struct_Disc_List* ptr_disc_obj = (C_struct_Disc_List*)kludge;
+    C_struct_Disc_List* ptr_disc_obj;
     ObjRef disc_class_ref;
     ObjRef object_class;
     ReferenceList* ref_list;
@@ -347,15 +348,15 @@ bool nwos_find_matching_disc_list(ObjRef files[MAX_FILES_PER_DISC_LIST], ObjRef*
 
     /* printf("num_refs: %d\n", num_refs); */
 
+    ptr_disc_obj = nwos_malloc(MAX_SIZE_DISC_LIST);
+
     for (i = 0; i < num_refs; i++)
     {
 	nwos_get_object_class(&ref_list->references[i], &object_class);
 
 	if (is_same_object(&object_class, &disc_class_ref))
 	{
-	    assert(nwos_read_variable_sized_object_from_disk(&ref_list->references[i], kludge, sizeof(kludge), &nwos_get_disc_list_object_size));
-
-	    /* remember ptr_disc_obj points to the kludge buffer */
+	    assert(nwos_read_variable_sized_object_from_disk(&ref_list->references[i], ptr_disc_obj, MAX_SIZE_DISC_LIST, &nwos_get_disc_list_object_size));
 
 	    if (nwos_decode_variable_sized_count(ptr_disc_obj->count) == count)   /* found a possible match */
 	    {
@@ -381,6 +382,9 @@ bool nwos_find_matching_disc_list(ObjRef files[MAX_FILES_PER_DISC_LIST], ObjRef*
 	}
     }
 
+    nwos_free(ptr_disc_obj);
+    ptr_disc_obj = NULL;
+
     nwos_free_reference_list(ref_list);
     ref_list = NULL;
 
@@ -393,8 +397,7 @@ bool nwos_find_matching_disc_list(ObjRef files[MAX_FILES_PER_DISC_LIST], ObjRef*
 
 ObjCreateResult nwos_create_disc_list(char id[12], ObjRef files[MAX_FILES_PER_DISC_LIST], ObjRef* ref)
 {
-    uint8 kludge[MAX_SIZE_DISC_LIST];
-    C_struct_Disc_List* ptr_disc_obj = (C_struct_Disc_List*)kludge;
+    C_struct_Disc_List* ptr_disc_obj;
     ObjRef disc_class_ref;
     ObjCreateResult result = FOUND_EXISTING;
     int i;
@@ -405,9 +408,9 @@ ObjCreateResult nwos_create_disc_list(char id[12], ObjRef files[MAX_FILES_PER_DI
 
     if (!nwos_find_disc_list(id, ref))   /* didn't find it */
     {
-	memset(kludge, 0, sizeof(kludge));  /* zero it out */
+	ptr_disc_obj = nwos_malloc(MAX_SIZE_DISC_LIST);
 
-	/* remember ptr_path_obj points to the kludge buffer */
+	memset(ptr_disc_obj, 0, MAX_SIZE_DISC_LIST);  /* zero it out */
 
 	nwos_find_or_create_private_class_definition("DISC LIST", &disc_class_ref);
 
@@ -438,11 +441,12 @@ ObjCreateResult nwos_create_disc_list(char id[12], ObjRef files[MAX_FILES_PER_DI
 
 	nwos_crc32_calculate((uint8*) &ptr_disc_obj->id, nwos_get_disc_list_object_size(ptr_disc_obj) - sizeof(EveryObject), ptr_disc_obj->header.common.data_chksum);
 
-	nwos_write_object_to_disk(ref, kludge, nwos_get_disc_list_object_size(ptr_disc_obj));
+	nwos_write_object_to_disk(ref, ptr_disc_obj, nwos_get_disc_list_object_size(ptr_disc_obj));
 
 	nwos_add_to_references(ref, &disc_class_ref);
 
 #ifdef VERIFY_WRITE
+	// WARNING: this code will no longer work since we don't have the kludge buffer anymore
 	ptr_disc_obj = malloc(sizeof(C_struct_Disc_List) + count * sizeof(ObjRef));
 	assert(nwos_read_object_from_disk(ref, ptr_disc_obj, sizeof(C_struct_Disc_List) + count * sizeof(ObjRef)));
 	assert(memcmp(kludge, ptr_disc_obj, sizeof(C_struct_Disc_List) + count * sizeof(ObjRef)) == 0);
@@ -454,6 +458,8 @@ ObjCreateResult nwos_create_disc_list(char id[12], ObjRef files[MAX_FILES_PER_DI
 	free(ptr_disc_obj);
 	ptr_disc_obj = NULL;
 #endif
+	nwos_free(ptr_disc_obj);
+	ptr_disc_obj = NULL;
 
 	result = CREATED_NEW;
     }
@@ -640,6 +646,87 @@ ObjCreateResult nwos_create_disc_copy(ObjRef* disc_list, int copy_num, ObjRef* l
 }
 
 
+/* Returns true if the file is stored on some backup media */
+
+bool nwos_file_has_backup(ObjRef* assoc_ref)
+{
+    ObjRef disc_list_class_ref;
+    ObjRef disc_copy_class_ref;
+    ObjRef class_ref;
+    C_struct_Path_And_File_Association assoc_obj;
+    C_struct_Disc_Copy copy_obj;
+    C_struct_Disc_List* ptr_disc_obj;
+    ReferenceList* assoc_ref_list_ptr;
+    ReferenceList* disc_ref_list_ptr;
+    int num_assoc_refs;
+    int num_disc_refs;
+    int i;
+    int j;
+    int count;
+
+    if (!nwos_find_private_class_definition("DISC LIST", &disc_list_class_ref)) return false;
+    if (!nwos_find_private_class_definition("DISC COPY", &disc_copy_class_ref)) return false;
+
+    ptr_disc_obj = nwos_malloc(MAX_SIZE_DISC_LIST);
+
+    assert(nwos_read_object_from_disk(assoc_ref, &assoc_obj, sizeof(assoc_obj)));  /* read the path and file assoc. object */
+
+    assoc_ref_list_ptr = nwos_malloc_reference_list(&assoc_obj.header.object.references);
+
+    num_assoc_refs = assoc_ref_list_ptr->common_header.num_refs;
+
+    for (i = 0; i < num_assoc_refs; i++)
+    {
+	nwos_get_object_class(&assoc_ref_list_ptr->references[i], &class_ref);
+
+	if (is_same_object(&class_ref, &disc_list_class_ref))
+	{
+	    assert(nwos_read_variable_sized_object_from_disk(&assoc_ref_list_ptr->references[i], ptr_disc_obj, MAX_SIZE_DISC_LIST, &nwos_get_disc_list_object_size));
+
+	    disc_ref_list_ptr = nwos_malloc_reference_list(&ptr_disc_obj->header.object.references);
+
+	    /* as long as we had to read the object, verify that this file is in the list */
+
+	    count = nwos_decode_variable_sized_count(ptr_disc_obj->count);
+	    for (j = 0; j < count; j++)
+	    {
+		if (is_same_object(assoc_ref, &ptr_disc_obj->files[j])) break;
+	    }
+	    assert(j < count);   /* something very wrong if it wasn't in the list */
+
+	    /* now find any disc copies that exist */
+
+	    num_disc_refs = disc_ref_list_ptr->common_header.num_refs;
+
+	    for (j = 0; j < num_disc_refs; j++)
+	    {
+		nwos_get_object_class(&disc_ref_list_ptr->references[j], &disc_copy_class_ref);
+
+		if (is_same_object(&class_ref, &disc_list_class_ref))
+		{
+		    assert(nwos_read_object_from_disk(&disc_ref_list_ptr->references[j], &copy_obj, sizeof(copy_obj)));
+		    assert(is_same_object(&assoc_ref_list_ptr->references[i], &copy_obj.disc_list));
+		    break;
+		}
+	    }
+	    
+	    nwos_free_reference_list(disc_ref_list_ptr);
+	    disc_ref_list_ptr = NULL;
+
+	    if (j < num_disc_refs) break;  /* found a disc_copy in references */
+	}
+    }
+
+    nwos_free_reference_list(assoc_ref_list_ptr);
+    assoc_ref_list_ptr = NULL;
+
+    nwos_free(ptr_disc_obj);
+    ptr_disc_obj = NULL;
+
+    return i < num_assoc_refs;;
+}
+
+
 
 /*-------------------------------------------------------------------------------------------------------------------*/
 /* Storage Location object stuff */
