@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2011-11-08 09:41:50 -0700 (Tue, 08 Nov 2011) $
+--   $Revision: 4939 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -550,6 +550,16 @@ bool nwos_find_private_name(const char* name, ObjRef* ref)
 }
 
 
+bool nwos_find_name(const char* name, ObjRef* ref)
+{
+#ifdef PUBLIC_MODE
+    return nwos_find_public_name(name, ref);
+#else
+    return nwos_find_private_name(name, ref);
+#endif
+}
+
+
 /* NOTE: the find part could be extracted from the create name below, but */
 /* I'm not sure exactly how to do it right now so I am leaving it as is.  */
 
