@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---         Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  QRW Software
+--   Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2011-07-06 21:33:05 -0600 (Wed, 06 Jul 2011) $
+--   $Revision: 4912 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -64,6 +64,12 @@
 #include "storage.h"
 
 
+#define MAX_TEMPORARY_OBJECTS 1024
+
+static void* temporary_objects[MAX_TEMPORARY_OBJECTS];
+
+
+
 /* Defined in file.c, not for general usage */
 
 extern void nwos_update_file_001_object_to_current(void* object, size_t size);
@@ -88,7 +94,8 @@ static bool object_header_checksum_matches(ObjectHeader* obj_header, uint8 store
 
     nwos_crc32_calculate((uint8*)obj_header, sizeof(*obj_header), computed);
 
-    return (stored[0] == computed[0] && stored[1] == computed[1] && stored[2] == computed[2] && stored[3] == computed[3]);
+//    return (stored[0] == computed[0] && stored[1] == computed[1] && stored[2] == computed[2] && stored[3] == computed[3]);
+    return (memcmp(stored, computed, sizeof(computed)) == 0);
 }
 
 
@@ -136,7 +143,7 @@ typedef struct
 
 bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 {
-    EveryObject* header;
+    EveryObject*  header;
     uint8 ivec[IVEC_SIZE];
     uint8* ptr_obj;
     Extra_Data_Block extra;
@@ -157,75 +164,91 @@ bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 
     assert(size > sizeof(CommonHeader));  /* make sure this wasn't called to just get the header */
 
-    memset(ivec, 0, sizeof(ivec));
-
-    bytes_remaining = size;
-
-    ptr_obj = object;
-
-    if (!nwos_read_block_from_disk_and_decrypt(ref, &extra, sizeof(extra), ivec, seq)) return false;
-
-    assert(sizeof(ObjRef) == 4);
-
-    header = (EveryObject*) &extra;
-
-    /* check_common_header(ref, &header->common);     -- this doesn't do anything right now */
-
-    if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* could be old encryption */
+    if (nwos_reference_type(ref) == Temporary_Reference)
     {
-#ifdef PUBLIC_MODE
-	return false;
-#else
-	memset(ivec, 0, sizeof(ivec));
-	if (!nwos_read_block_from_disk_and_old_decrypt(ref, &extra, sizeof(extra), ivec, seq)) return false;
+	i = (int) (nwos_ref_to_word(ref) - MINIMUM_TEMPORARY_REFERENCE);
 
-	if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* neither decryption worked */
+	assert(0 <= i && i < MAX_TEMPORARY_OBJECTS);
+
+	memcpy(object, temporary_objects[i], size);
+
+	header = (EveryObject*) object;
+
+	if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* could be old encryption */
 	{
 	    return false;
 	}
-
-	use_old_decryption = true;
-#endif
     }
-
-    *ptr_obj++ = extra.flags[0];
-    *ptr_obj++ = extra.flags[1];
-    *ptr_obj++ = extra.flags[2];
-    *ptr_obj++ = extra.flags[3];
-    *ptr_obj++ = extra.id.id[0];
-    *ptr_obj++ = extra.id.id[1];
-    *ptr_obj++ = extra.id.id[2];
-    *ptr_obj++ = extra.id.id[3];
-
-
-    while (bytes_remaining > FILE_BLOCK_SIZE)
+    else
     {
-	for (i = 0; i < SIZE_EXTRA_BLOCK_DATA; i++)
+	memset(ivec, 0, sizeof(ivec));
+
+	bytes_remaining = size;
+
+	ptr_obj = object;
+
+	if (!nwos_read_block_from_disk_and_decrypt(ref, &extra, sizeof(extra), ivec, seq)) return false;
+
+	assert(sizeof(ObjRef) == 4);
+
+	header = (EveryObject*) &extra;
+
+	/* check_common_header(ref, &header->common);     -- this doesn't do anything right now */
+
+	if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* could be old encryption */
 	{
-	    *ptr_obj++ = extra.data[i];
-	    bytes_remaining--;
+#ifdef PUBLIC_MODE
+	    return false;
+#else
+	    memset(ivec, 0, sizeof(ivec));
+	    if (!nwos_read_block_from_disk_and_old_decrypt(ref, &extra, sizeof(extra), ivec, seq)) return false;
+
+	    if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* neither decryption worked */
+	    {
+		return false;
+	    }
+
+	    use_old_decryption = true;
+#endif
 	}
 
-	/* save the reference because the extra block is going to get written over, we don't want it to change */
-	copy_reference(&next_ref, &extra.next_block);
+	*ptr_obj++ = extra.flags[0];
+	*ptr_obj++ = extra.flags[1];
+	*ptr_obj++ = extra.flags[2];
+	*ptr_obj++ = extra.flags[3];
+	*ptr_obj++ = extra.id.id[0];
+	*ptr_obj++ = extra.id.id[1];
+	*ptr_obj++ = extra.id.id[2];
+	*ptr_obj++ = extra.id.id[3];
 
-	seq++;
+	while (bytes_remaining > FILE_BLOCK_SIZE)
+	{
+	    for (i = 0; i < SIZE_EXTRA_BLOCK_DATA; i++)
+	    {
+		*ptr_obj++ = extra.data[i];
+		bytes_remaining--;
+	    }
 
-	if (use_old_decryption)
-	{
-	    if (!nwos_read_block_from_disk_and_old_decrypt(&next_ref, &extra, sizeof(extra), ivec, seq)) return false;
-	}
-	else
-	{
-	    if (!nwos_read_block_from_disk_and_decrypt(&next_ref, &extra, sizeof(extra), ivec, seq)) return false;
+	    /* save the reference because the extra block is going to get written over, we don't want it to change */
+	    copy_reference(&next_ref, &extra.next_block);
+
+	    seq++;
+
+	    if (use_old_decryption)
+	    {
+		if (!nwos_read_block_from_disk_and_old_decrypt(&next_ref, &extra, sizeof(extra), ivec, seq)) return false;
+	    }
+	    else
+	    {
+		if (!nwos_read_block_from_disk_and_decrypt(&next_ref, &extra, sizeof(extra), ivec, seq)) return false;
+	    }
 	}
 
+	assert(bytes_remaining <= FILE_BLOCK_SIZE);
+
+	for (i = 0; i < bytes_remaining - 8; i++) *ptr_obj++ = extra.data[i];
     }
 
-    assert(bytes_remaining <= FILE_BLOCK_SIZE);
-
-    for (i = 0; i < bytes_remaining - 8; i++) *ptr_obj++ = extra.data[i];
-
 #if 0
     printf("read obj path: %s\n", path);
     printf("read obj pointer: %p\n", object);
@@ -281,8 +304,8 @@ bool nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t
     size_t bytes_remaining;
     int i;
     int seq = 0;
-    size_t xyzzy;
-    char msg[128];
+//    size_t xyzzy;
+//    char msg[128];
     bool use_old_decryption = false;
 
     assert(!is_void_reference(ref));
@@ -333,15 +356,15 @@ bool nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t
     *ptr_obj++ = extra.id.id[3];
 
     bytes_remaining = (*size_function)(&extra);
-    xyzzy = nwos_get_object_size(&extra);
+    //xyzzy = nwos_get_object_size(&extra);
 
     //    assert(xyzzy == 0 || bytes_remaining == xyzzy);
-    if (xyzzy != 0 && bytes_remaining != xyzzy)
-    {
-	snprintf(msg, sizeof(msg), "Warning: size_function: %zd  get_object_size: %zd", bytes_remaining, xyzzy);
-	nwos_log(msg);
-	printf("%s\n", msg);
-    }
+    //if (xyzzy != 0 && bytes_remaining != xyzzy)
+    //{
+    //	snprintf(msg, sizeof(msg), "Warning: size_function: %zd  get_object_size: %zd", bytes_remaining, xyzzy);
+    //	nwos_log(msg);
+    //	printf("%s\n", msg);
+    //}
 
     assert(bytes_remaining <= max_size);
 
@@ -589,43 +612,55 @@ void nwos_get_object_class_without_update(ObjRef* obj, ObjRef* object_class)
 {
     EveryObject header;
     uint8 ivec[IVEC_SIZE];
+    int i;
 
     assert(!is_void_reference(obj));
 
-    memset(ivec, 0, sizeof(ivec));
-
-    assert(nwos_read_block_from_disk_and_decrypt(obj, &header, sizeof(header), ivec, 0));
-
-    /* check_common_header(obj, &header.common);  -- this doesn't currently do anything */
-
-    /* if it is not a reference list check the header checksum */
-
-    if (!is_same_object(&header.common.class_definition, &nwos_reference_list_class_ref))
+    if (nwos_reference_type(obj) == Temporary_Reference)
     {
-	if (!object_header_checksum_matches(&header.object, header.common.header_chksum))   /* could be old encryption */
-	{
-#ifdef PUBLIC_MODE
-	    printf("\n");
-	    fflush(stdout);
-	    fprintf(stderr, "Unable to decrypt object class without update: %08x\n", nwos_ref_to_word(obj));
-	    nwos_terminate_objectify();
-	    exit(1);
-#else
-	    memset(ivec, 0, sizeof(ivec));
-	    assert(nwos_read_block_from_disk_and_old_decrypt(obj, &header, sizeof(header), ivec, 0));
+	i = (int) (nwos_ref_to_word(obj) - MINIMUM_TEMPORARY_REFERENCE);
 
-	    if (!is_same_object(&header.common.class_definition, &nwos_reference_list_class_ref))
+	assert(0 <= i && i < MAX_TEMPORARY_OBJECTS);
+
+	memcpy(&header, temporary_objects[i], sizeof(header));
+    }
+    else
+    {
+	memset(ivec, 0, sizeof(ivec));
+
+	assert(nwos_read_block_from_disk_and_decrypt(obj, &header, sizeof(header), ivec, 0));
+
+	/* check_common_header(obj, &header.common);  -- this doesn't currently do anything */
+
+	/* if it is not a reference list check the header checksum */
+
+	if (!is_same_object(&header.common.class_definition, &nwos_reference_list_class_ref))
+	{
+	    if (!object_header_checksum_matches(&header.object, header.common.header_chksum))   /* could be old encryption */
 	    {
-		if (!object_header_checksum_matches(&header.object, header.common.header_chksum))   /* neither decryption worked */
+#ifdef PUBLIC_MODE
+		printf("\n");
+		fflush(stdout);
+		fprintf(stderr, "Unable to decrypt object class without update: %08x\n", nwos_ref_to_word(obj));
+		nwos_terminate_objectify();
+		exit(1);
+#else
+		memset(ivec, 0, sizeof(ivec));
+		assert(nwos_read_block_from_disk_and_old_decrypt(obj, &header, sizeof(header), ivec, 0));
+
+		if (!is_same_object(&header.common.class_definition, &nwos_reference_list_class_ref))
 		{
-		    printf("\n");
-		    fflush(stdout);
-		    fprintf(stderr, "Unable to decrypt object class without update: %08x\n", nwos_ref_to_word(obj));
-		    nwos_terminate_objectify();
-		    exit(1);
+		    if (!object_header_checksum_matches(&header.object, header.common.header_chksum))   /* neither decryption worked */
+		    {
+			printf("\n");
+			fflush(stdout);
+			fprintf(stderr, "Unable to decrypt object class without update: %08x\n", nwos_ref_to_word(obj));
+			nwos_terminate_objectify();
+			exit(1);
+		    }
 		}
-	    }
 #endif
+	    }
 	}
     }
 
@@ -663,4 +698,65 @@ void nwos_fill_in_common_header(CommonHeader* common, ObjRef* ref, ObjRef* class
 }
 
 
+void* nwos_malloc_temporary_object(size_t size, ObjRef* temp_ref)
+{
+    int i;
+
+    for (i = 0; i < MAX_TEMPORARY_OBJECTS; i++)
+    {
+	if (temporary_objects[i] == NULL) break;
+    }
+
+    assert(i < MAX_TEMPORARY_OBJECTS);
+
+    temporary_objects[i] = nwos_malloc(size);
+
+    nwos_word_to_ref(MINIMUM_TEMPORARY_REFERENCE + (uint32)i, temp_ref);
+
+//    printf("i: %08x  temp_ref: %08x\n", MINIMUM_TEMPORARY_REFERENCE + (uint32)i, nwos_ref_to_word(temp_ref) );
+
+    return temporary_objects[i];
+}
+
+
+
+static void* common_free_temporary_object(ObjRef* ref)
+{
+    int i;
+    uint32 uint32ref = nwos_ref_to_word(ref);
+    void* object;
+
+//    printf("uint32ref: %08x\n", uint32ref);
+
+    assert(MINIMUM_TEMPORARY_REFERENCE <= uint32ref && uint32ref <= MAXIMUM_TEMPORARY_REFERENCE);
+
+    i = (int) (uint32ref - MINIMUM_TEMPORARY_REFERENCE);
+
+    assert(0 <= i && i < MAX_TEMPORARY_OBJECTS);
+
+    object = temporary_objects[i];
+
+    temporary_objects[i] = NULL;
+
+    nwos_free(object);
+
+    return object;   /* This value MUST only be used for verification in the nwos_free_temporary_object function!! */
+}
+
+
+void nwos_free_temporary_object_from_reference(ObjRef* ref)
+{
+    common_free_temporary_object(ref);
+}
+
+
+void nwos_free_temporary_object(void* object)
+{
+    void* verify;
+
+    verify = common_free_temporary_object(&((EveryObject*)object)->common.id);
+
+    assert(verify == object);
+}
+
 
