@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---            Copyright (C) 2005, 2006, 2007, 2008, 2009  QRW Software
+--         Copyright (C) 2005, 2006, 2007, 2008, 2009, 2011  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2011-11-28 14:40:21 -0700 (Mon, 28 Nov 2011) $
+--   $Revision: 4970 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -379,6 +379,7 @@ void nwos_add_person()
     char maiden_name[MAX_NAME_SIZE];
     char nickname[MAX_NAME_SIZE];
     char birth_date[MAX_DATE_SIZE];
+    char death_date[MAX_DATE_SIZE];
     char temp[MAX_NAME_SIZE];
     ObjRef person_class_ref;
     ObjRef last_name_ref;
@@ -387,14 +388,19 @@ void nwos_add_person()
     ObjRef object_class;
     EveryObject name_header;
     ReferenceList* ref_list;
-    int month, day, year;
+    int born_month, born_day, born_year;
+    int died_month, died_day, died_year;
     int num_refs;
     int i;
     bool ok;
     bool printed_warning = false;
     C_struct_Person person_obj;
 
+#ifdef PUBLIC_MODE
+    nwos_find_public_class_definition("PERSON", &person_class_ref);
+#else
     nwos_find_or_create_private_class_definition("PERSON", &person_class_ref);
+#endif
 
     ok = false;
     while (!ok)
@@ -462,20 +468,20 @@ void nwos_add_person()
 	}
 	else
 	{
-	    ok = (sscanf(birth_date, "%d/%d/%d", &month, &day, &year) == 3);
-	    if (!ok) ok = (sscanf(birth_date, "%d-%d-%d", &month, &day, &year) == 3);
-	    if (ok) ok = (1582 <= year && year <= 9999);    /* 1582 start of Gregorian calendar */
-	    if (ok) ok = (1 <= month && month <= 12);
-	    if (ok) switch (month)
+	    ok = (sscanf(birth_date, "%d/%d/%d", &born_month, &born_day, &born_year) == 3);
+	    if (!ok) ok = (sscanf(birth_date, "%d-%d-%d", &born_month, &born_day, &born_year) == 3);
+	    if (ok) ok = (1582 <= born_year && born_year <= 9999);    /* 1582 start of Gregorian calendar */
+	    if (ok) ok = (1 <= born_month && born_month <= 12);
+	    if (ok) switch (born_month)
 	    {
 	      case 2:
-		if (nwos_is_leap_year(year))
+		if (nwos_is_leap_year(born_year))
 		{
-		    ok = (day <= 29);
+		    ok = (born_day <= 29);
 		}
 		else
 		{
-		    ok = (day <= 28);
+		    ok = (born_day <= 28);
 		}
 		break;
 
@@ -483,19 +489,60 @@ void nwos_add_person()
 	      case 6:
 	      case 9:
 	      case 11:
-		ok = (day <= 30);
+		ok = (born_day <= 30);
 		break;
 
 	      default:
-		ok = (day <= 31);
+		ok = (born_day <= 31);
 		break;
 	    }
 	}
     }
 
-    if (nwos_find_private_name(last_name, &last_name_ref) && nwos_find_private_name(first_name, &first_name_ref))
+    ok = false;
+    while (!ok)
     {
-	if (*middle_name == '\0' || !nwos_find_private_name(middle_name, &middle_name_ref))
+	nwos_ask_user("death date M/D/YYYY", death_date, sizeof(death_date));
+	if (*death_date == '\0')
+	{
+	    ok = true;    /* none given */
+	}
+	else
+	{
+	    ok = (sscanf(death_date, "%d/%d/%d", &died_month, &died_day, &died_year) == 3);
+	    if (!ok) ok = (sscanf(death_date, "%d-%d-%d", &died_month, &died_day, &died_year) == 3);
+	    if (ok) ok = (1582 <= died_year && died_year <= 9999);    /* 1582 start of Gregorian calendar */
+	    if (ok) ok = (1 <= died_month && died_month <= 12);
+	    if (ok) switch (died_month)
+	    {
+	      case 2:
+		if (nwos_is_leap_year(died_year))
+		{
+		    ok = (died_day <= 29);
+		}
+		else
+		{
+		    ok = (died_day <= 28);
+		}
+		break;
+
+	      case 4:
+	      case 6:
+	      case 9:
+	      case 11:
+		ok = (died_day <= 30);
+		break;
+
+	      default:
+		ok = (died_day <= 31);
+		break;
+	    }
+	}
+    }
+
+    if (nwos_find_name(last_name, &last_name_ref) && nwos_find_name(first_name, &first_name_ref))
+    {
+	if (*middle_name == '\0' || !nwos_find_name(middle_name, &middle_name_ref))
 	{
 	    void_reference(&middle_name_ref);
 	}
@@ -556,7 +603,11 @@ void nwos_add_person()
 
 	nwos_fill_in_common_header(&person_obj.header.common, &reference, &person_class_ref);
 
+#ifdef PUBLIC_MODE
+	nwos_find_public_gender(gender, &person_obj.gender);
+#else
 	nwos_find_or_create_private_gender(gender, &person_obj.gender);
+#endif
 
 	nwos_create_name(first_name, &person_obj.first_name);
 
@@ -584,7 +635,12 @@ void nwos_add_person()
 
 	if (*birth_date != '\0')
 	{
-	    nwos_create_date(year, month, day, &person_obj.birth_date);
+	    nwos_create_date(born_year, born_month, born_day, &person_obj.birth_date);
+	}
+
+	if (*death_date != '\0')
+	{
+	    nwos_create_date(died_year, died_month, died_day, &person_obj.death_date);
 	}
 
 	nwos_create_reference_list(&reference, &person_obj.header.object.references);
@@ -625,6 +681,11 @@ void nwos_add_person()
 	    nwos_add_to_references(&reference, &person_obj.birth_date);
 	}
 	
+	if (!is_void_reference(&person_obj.death_date))
+	{
+	    nwos_add_to_references(&reference, &person_obj.death_date);
+	}
+	
 	printf("Object created: %02x%02x%02x%02x\n", reference.id[0], reference.id[1], reference.id[2], reference.id[3]);
     }
 }
@@ -753,7 +814,6 @@ bool nwos_any_persons_named(char* name)
     ObjRef object_class;
     EveryObject name_header;
     ObjRef person_class_ref;
-    int person_count;
     ReferenceList* ref_list;
     int num_refs;
     int i;
@@ -773,8 +833,6 @@ bool nwos_any_persons_named(char* name)
 
 	/* printf("name num refs: %d\n", num_refs); */
 
-	person_count = 0;
-
 	for (i = 0; i < num_refs; i++)
 	{
 	    nwos_get_object_class(&ref_list->references[i], &object_class);   /* find out what kind of object it is */
@@ -923,7 +981,11 @@ void nwos_list_persons()
     int i;
     char name[32];
 
+#ifdef PUBLIC_MODE
+    if (!nwos_find_public_class_definition("PERSON", &person_class_ref))
+#else
     if (!nwos_find_private_class_definition("PERSON", &person_class_ref))
+#endif
     {
 	printf("\nThere are no persons in the system!\n\n");
 	return;
@@ -1025,6 +1087,13 @@ void nwos_describe_person(char* name)
 	    nwos_date_to_string(&person_obj.birth_date, temp, sizeof(temp));
 	    printf("%s\n", temp);
 	}
+
+	if (!is_void_reference(&person_obj.death_date))
+	{
+	    printf("died: ");
+	    nwos_date_to_string(&person_obj.death_date, temp, sizeof(temp));
+	    printf("%s\n", temp);
+	}
     }
 
     printf("\n");
