@@ -1,6 +1,6 @@
 /*
 --          This file is part of the New World OS and Objectify projects
---         Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  QRW Software
+--   Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011  QRW Software
 --               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --
 --   This program is free software: you can redistribute it and/or modify
@@ -39,8 +39,8 @@
 --      Salt Lake City, UT 84127-0511, USA.
 --
 --   $Author: jsedwards $
---   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
---   $Revision: 4384 $
+--   $Date: 2011-02-20 21:23:53 -0700 (Sun, 20 Feb 2011) $
+--   $Revision: 4870 $
 --
 --   NOTE: Subversion does not support the Log keyword so I have removed the
 --   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -50,6 +50,7 @@
 --
 */
 
+#include <limits.h>                    /* define PATH_MAX */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -96,6 +97,7 @@ typedef struct {
   int num_refs;
   Ref_List_First_Block* first_block;
   Ref_List_Extra_Block* last_block;        /* null if only one block in list */
+  Sorted_Reference_List*  sorted_list;
   int age;
 } Block_Cache;
 
@@ -151,6 +153,7 @@ static void read_ref_list_into_cache(Block_Cache* cache)
     cache->first_block->next_block_ptr = NULL;
 
     cache->last_block = NULL;
+    cache->sorted_list = NULL;
 
     while (1)
     {
@@ -179,18 +182,19 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 	    {
 		break;   /* found correct decryption, exit loop */
 	    }
-
-	    if (nwos_reference_type(&cache->ref) == Public_Reference || use_old_decryption)
-	    {
-		printf("\n");
-		fflush(stdout);
-		fprintf(stderr, "Unable to decrypt reference list: %08x\n", nwos_ref_to_word(&cache->ref));
-		nwos_terminate_objectify();
-		exit(1);
-	    }
 	}
 
-	use_old_decryption = true;
+	if (nwos_reference_type(&cache->ref) == Public_Reference || use_old_decryption)
+	{
+	    printf("\n");
+	    fflush(stdout);
+	    fprintf(stderr, "Unable to decrypt reference list: %08x\n", nwos_ref_to_word(&cache->ref));
+	    exit(1);
+	}
+	else
+	{
+	    use_old_decryption = true;
+	}
     }
 
 
@@ -279,6 +283,7 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
 static void write_reference_list(Block_Cache* cache)
 {
+    uint8 ivec[IVEC_SIZE];
     Ref_List_Extra_Block *block = NULL;
     int running_total;
     int refs_remaining;
@@ -328,10 +333,6 @@ static void write_reference_list(Block_Cache* cache)
 
 	    seq = 0;
 	}
-	else                    /* we don't need to write this block so we can get rid of it now */
-	{
-	    nwos_free(cache->first_block);
-	}
     }
     else
     {
@@ -348,8 +349,6 @@ static void write_reference_list(Block_Cache* cache)
 
 	    assert(is_same_object(&prev_ptr->next_block_ref, &next_ptr->id));  /* next pointer ref matches */
 
-	    nwos_free(prev_ptr);   /* won't be needing this anymore */
-
 	    refs_remaining = cache->num_refs - running_total;
 
 	    if (refs_remaining <= MAX_REFS_IN_SIDECAR)
@@ -392,7 +391,9 @@ static void write_reference_list(Block_Cache* cache)
 	    }
 	    else
 	    {
+		assert(next_ptr != NULL);
 		assert(next_ptr->next_block_ptr != NULL); /* the next block pointer should not be null */
+
 		running_total += MAX_REFS_IN_SIDECAR;
 	    }
 	    
@@ -401,14 +402,6 @@ static void write_reference_list(Block_Cache* cache)
 	    seq++;
 	}
 
-
-	if (block == NULL)      /* we don't need to write this block so we can get rid of it now */
-	{
-	    assert(next_ptr != NULL);
-
-	    nwos_free(next_ptr);
-	}
-
 	assert(running_total == cache->num_refs);
 	assert(cache->last_block == prev_ptr);
     }
@@ -424,15 +417,79 @@ static void write_reference_list(Block_Cache* cache)
 
 	/* printf("writing block: %02x%02x%02x%02x\n", ref.id[0], ref.id[1], ref.id[2], ref.id[3]); */
 
-	nwos_write_block_to_disk_and_encrypt(&ref, &block->dirty, FILE_BLOCK_SIZE, block->ivec, seq);
+	memcpy(ivec, block->ivec, sizeof(ivec));  /* make a copy of the ivec so the stored one isn't overwritten */
 
-	nwos_free(block);
+	nwos_write_block_to_disk_and_encrypt(&ref, &block->dirty, FILE_BLOCK_SIZE, ivec, seq);
+    }
+}
+
+
+static void free_reference_list(Block_Cache* cache)
+{
+    int running_total;
+    int refs_remaining;
+    Ref_List_Extra_Block* prev_ptr = NULL;
+    Ref_List_Extra_Block* next_ptr = NULL;
+
+    if (cache->num_refs <= MAX_REFS_IN_REF_LIST)    /* there is just one block */
+    {
+	assert(cache->first_block->next_block_ptr == NULL);  /* the next block pointer should be null */
+	assert(is_void_reference(&cache->first_block->refs[cache->num_refs]));
+	assert(cache->last_block == NULL);
+	assert(cache->first_block->list.common_header.flags != 0xffffffff);    /* block should not be dirty */
+
+	nwos_free(cache->first_block);
+    }
+    else
+    {
+	assert(cache->first_block->next_block_ptr != NULL);  /* the next block pointer should not be null */
+
+	running_total = MAX_REFS_IN_REF_LIST;
+	prev_ptr = (Ref_List_Extra_Block*)cache->first_block;
+
+	while (running_total < cache->num_refs)    /* read more blocks */
+	{
+	    next_ptr = prev_ptr->next_block_ptr;
+
+	    assert(is_same_object(&prev_ptr->next_block_ref, &next_ptr->id));  /* next pointer ref matches */
+
+	    nwos_free(prev_ptr);   /* won't be needing this anymore */
+
+	    refs_remaining = cache->num_refs - running_total;
+
+	    if (refs_remaining <= MAX_REFS_IN_SIDECAR)
+	    {
+		assert(next_ptr->next_block_ptr == NULL);  /* the next block pointer should be null */
+		assert(is_void_reference(&next_ptr->refs[refs_remaining]));
+		assert(next_ptr->dirty != -1);    /* no blocks should be dirty dirty */
+
+		running_total += refs_remaining;
+	    }
+	    else
+	    {
+		assert(next_ptr->next_block_ptr != NULL); /* the next block pointer should not be null */
+		running_total += MAX_REFS_IN_SIDECAR;
+	    }
+	    
+	    prev_ptr = next_ptr;
+	}
+
+	nwos_free(next_ptr);
+
+	assert(running_total == cache->num_refs);
+	assert(cache->last_block == prev_ptr);
     }
 
     /* just in case, nuke these */
     cache->first_block = NULL;
     cache->last_block = NULL;
     memset(&cache->ref, 0, sizeof(cache->ref));
+
+    if (cache->sorted_list != NULL)
+    {
+	nwos_free(cache->sorted_list);
+	cache->sorted_list = NULL;
+    }
 }
 
 
@@ -489,6 +546,218 @@ static Block_Cache* find_ref_list_in_cache(ObjRef* ref)
 }
 
 
+#ifndef USE_PREDEFINED_STRUCTS
+
+static size_t get_class_object_size(void* class_obj)
+{
+    return sizeof(C_struct_Class_Definition) + (((C_struct_Class_Definition*)class_obj)->count * sizeof(ObjRef));
+}
+
+
+static size_t get_file_path_object_size(void* file_path_obj)
+{
+    return sizeof(C_struct_File_Path) + (((C_struct_File_Path*)file_path_obj)->count);
+}
+
+
+static bool less_than(ObjRef* left_ref, ObjRef* right_ref)
+{
+    bool result = false;
+    bool no_compare_function_defined_for_class = false;
+    ObjRef left_class_ref;
+    ObjRef right_class_ref;
+    uint8 kludge[MAX_CLASS_DEFINITION_OBJ_SIZE];
+    C_struct_Class_Definition* class_def_obj_ptr = (C_struct_Class_Definition*)kludge;
+    int32 public_class;
+
+    nwos_get_object_class(left_ref, &left_class_ref);
+    nwos_get_object_class(right_ref, &right_class_ref);
+
+    assert(is_same_object(&left_class_ref, &right_class_ref));
+
+
+    if (nwos_reference_type(left_ref) == Temporary_Reference)       /* can't use left to decide */
+    {
+	assert(nwos_reference_type(right_ref) != Temporary_Reference);    /* can't compare two temporaries */ 
+
+	if (nwos_reference_type(right_ref) == Public_Reference)
+	{
+	    public_class = nwos_ref_to_word(&right_class_ref);
+	}
+	else
+	{
+	    assert(nwos_read_variable_sized_object_from_disk(&right_class_ref, kludge, sizeof(kludge), &get_class_object_size));
+	    public_class = nwos_ref_to_word(&class_def_obj_ptr->header.object.clone_of);
+	}
+    }
+    else
+    {
+	if (nwos_reference_type(left_ref) == Public_Reference)
+	{
+	    public_class = nwos_ref_to_word(&left_class_ref);
+	}
+	else
+	{
+	    assert(nwos_read_variable_sized_object_from_disk(&left_class_ref, kludge, sizeof(kludge), &get_class_object_size));
+	    public_class = nwos_ref_to_word(&class_def_obj_ptr->header.object.clone_of);
+	}
+    }
+
+    switch (public_class)
+    {
+      case NWOS_FILE_PATH_PUBLIC_REF:
+	{
+	    uint8 left_obj[sizeof(C_struct_File_Path) + PATH_MAX];
+	    uint8 right_obj[sizeof(C_struct_File_Path) + PATH_MAX];
+	    nwos_read_variable_sized_object_from_disk(left_ref, &left_obj, sizeof(left_obj), &get_file_path_object_size);
+	    nwos_read_variable_sized_object_from_disk(right_ref, &right_obj, sizeof(right_obj), &get_file_path_object_size);
+	    result = nwos_file_path_less_than((C_struct_File_Path*)left_obj, (C_struct_File_Path*)right_obj);
+	}
+	break;
+
+      case NWOS_FILE_PUBLIC_REF:
+	{
+	    C_struct_File left_obj;
+	    C_struct_File right_obj;
+	    nwos_read_object_from_disk(left_ref, &left_obj, sizeof(left_obj));
+	    nwos_read_object_from_disk(right_ref, &right_obj, sizeof(right_obj));
+	    result = nwos_file_less_than(&left_obj, &right_obj);
+	}
+	break;
+
+      case NWOS_MD5SUM_PUBLIC_REF:
+	{
+	    C_struct_MD5sum left_obj;
+	    C_struct_MD5sum right_obj;
+	    nwos_read_object_from_disk(left_ref, &left_obj, sizeof(left_obj));
+	    nwos_read_object_from_disk(right_ref, &right_obj, sizeof(right_obj));
+	    result = nwos_md5_less_than(&left_obj, &right_obj);
+	}
+	break;
+
+      case NWOS_SHA1SUM_PUBLIC_REF:
+	{
+	    C_struct_SHA1sum left_obj;
+	    C_struct_SHA1sum right_obj;
+	    nwos_read_object_from_disk(left_ref, &left_obj, sizeof(left_obj));
+	    nwos_read_object_from_disk(right_ref, &right_obj, sizeof(right_obj));
+	    result = nwos_sha1_less_than(&left_obj, &right_obj);
+	}
+	break;
+
+      case NWOS_SHA256SUM_PUBLIC_REF:
+	{
+	    C_struct_SHA256sum left_obj;
+	    C_struct_SHA256sum right_obj;
+	    nwos_read_object_from_disk(left_ref, &left_obj, sizeof(left_obj));
+	    nwos_read_object_from_disk(right_ref, &right_obj, sizeof(right_obj));
+	    result = nwos_sha256_less_than(&left_obj, &right_obj);
+	}
+	break;
+
+      case NWOS_SHA512SUM_PUBLIC_REF:
+	{
+	    C_struct_SHA512sum left_obj;
+	    C_struct_SHA512sum right_obj;
+	    nwos_read_object_from_disk(left_ref, &left_obj, sizeof(left_obj));
+	    nwos_read_object_from_disk(right_ref, &right_obj, sizeof(right_obj));
+	    result = nwos_sha512_less_than(&left_obj, &right_obj);
+	}
+	break;
+
+      default:
+	assert(no_compare_function_defined_for_class);
+	break;
+    }
+
+    return result;
+}
+
+
+static void insert_reference_into_sorted_list(ObjRef* ref, Block_Cache* cache)
+{
+    ObjRef object_class;
+    int lwr;
+    int upr;
+    int mid;
+    int i;
+
+
+    nwos_get_object_class(ref, &object_class);
+
+    if (is_same_object(&object_class, &cache->sorted_list->class))
+    {
+	/* first make sure there is room for more */
+
+	assert(cache->sorted_list->count <= cache->sorted_list->capacity);
+
+	if (cache->sorted_list->capacity == cache->sorted_list->count)
+	{
+	    cache->sorted_list->capacity = cache->sorted_list->capacity + 1024;
+
+	    cache->sorted_list = nwos_realloc(cache->sorted_list, sizeof(Sorted_Reference_List) + (cache->sorted_list->capacity * sizeof(ObjRef)));
+	}
+
+	lwr = 1;
+	mid = 0;
+	upr = cache->sorted_list->count;
+
+	while (lwr <= upr)
+	{
+	    mid = (upr + lwr) / 2 - 1;    /* mid == zero based index */
+
+	    if (less_than(ref, &cache->sorted_list->references[mid]))
+	    {
+		upr = mid;
+	    }
+	    else if (mid + 1 == cache->sorted_list->count)    /* greater than or equal and at the end */
+	    {
+		mid = mid + 1;
+		break;
+	    }
+	    else if (less_than(ref, &cache->sorted_list->references[mid + 1]))
+	    {
+		mid = mid + 1;
+		break;
+	    }
+	    else
+	    {
+		lwr = mid + 2;
+	    }
+	}
+
+	if (cache->sorted_list->count == 0)
+	{
+	    assert(mid == 0);
+	}
+	else
+	{
+	    assert(0 <= mid && mid <= cache->sorted_list->count);
+	}
+
+	for (i = cache->sorted_list->count; i > mid; i--) copy_reference(&cache->sorted_list->references[i], &cache->sorted_list->references[i-1]);
+
+	copy_reference(&cache->sorted_list->references[mid], ref);
+
+	cache->sorted_list->count++;
+
+#ifdef VERIFY_SORT
+	for (i = 0; i < cache->sorted_list->count - 1; i++)
+	{
+	    assert(!less_than(&cache->sorted_list->references[i+1], &cache->sorted_list->references[i]));
+	}
+#endif
+    }
+}
+#endif
+
+
+static size_t reference_list_size(Block_Cache* cache)
+{
+    return sizeof(ReferenceList) + (cache->num_refs * sizeof(ObjRef));
+}
+
+
 /* WARNING: this only works on reference lists */
 
 size_t nwos_reference_list_size(ObjRef* ref)
@@ -499,21 +768,16 @@ size_t nwos_reference_list_size(ObjRef* ref)
 
     cache = find_ref_list_in_cache(ref);
 
-    return sizeof(ReferenceList) + (cache->num_refs * sizeof(ObjRef));
+    return reference_list_size(cache);
 }
 
 
-void nwos_read_reference_list_from_disk(ObjRef* ref, ReferenceList* object, size_t size)
+static void read_reference_list_from_disk(Block_Cache* cache, ReferenceList* object, size_t size)
 {
-    Block_Cache* cache;
     int running_total;
     int refs_remaining;
     Ref_List_Extra_Block* next_ptr;
 
-    assert(!is_void_reference(ref));
-
-    cache = find_ref_list_in_cache(ref);
-
     assert(size >= sizeof(ReferenceList) + cache->num_refs * sizeof(ObjRef));
 
     /* copy the first block (which is a reference list object */
@@ -565,6 +829,18 @@ void nwos_read_reference_list_from_disk(ObjRef* ref, ReferenceList* object, size
 }
 
 
+void nwos_read_reference_list_from_disk(ObjRef* ref, ReferenceList* object, size_t size)
+{
+    Block_Cache* cache;
+
+    assert(!is_void_reference(ref));
+
+    cache = find_ref_list_in_cache(ref);
+
+    read_reference_list_from_disk(cache, object, size);
+}
+
+
 void nwos_create_reference_list_with_existing_id(ObjRef* for_obj, ObjRef* ref_list)
 {
     Block_Cache* cache;
@@ -884,6 +1160,13 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 
 	assert(running_total + refs_remaining + 1 == cache->num_refs);
     }
+
+#ifndef USE_PREDEFINED_STRUCTS
+    if (cache->sorted_list != NULL)
+    {
+	insert_reference_into_sorted_list(ref, cache);
+    }
+#endif
 }
 
 
@@ -1253,6 +1536,22 @@ void nwos_flush_dirty_ref_lists()
 }
 
 
+void nwos_free_ref_lists()
+{
+    int i;
+
+    nwos_flush_dirty_ref_lists();
+
+    for (i = 0; i < REF_LIST_CACHE_SIZE; i++)
+    {
+	if (!is_void_reference(&ref_list_cache[i].ref))
+	{
+	    free_reference_list(&ref_list_cache[i]);
+	}
+    }
+}
+
+
 ReferenceList* nwos_malloc_reference_list(ObjRef* ref)
 {
     size_t ref_list_size;
@@ -1355,3 +1654,88 @@ void nwos_free_reference_list(ReferenceList* ref_list)
     nwos_free(ref_list);
 }
 
+
+#ifndef USE_PREDEFINED_STRUCTS
+
+Sorted_Reference_List* nwos_get_sorted_reference_list(ObjRef* list_ref, ObjRef* class_ref)
+{
+    Block_Cache* cache;
+    ReferenceList* ref_list;
+    int i;
+
+    assert(!is_void_reference(list_ref));
+
+    cache = find_ref_list_in_cache(list_ref);
+
+    if (cache->sorted_list == NULL || !is_same_object(&cache->sorted_list->class, class_ref))
+    {
+	if (cache->sorted_list == NULL)
+	{
+	    cache->sorted_list = nwos_malloc(sizeof(Sorted_Reference_List) + (cache->num_refs * sizeof(ObjRef)));
+	}
+
+	copy_reference(&cache->sorted_list->class, class_ref);
+
+	cache->sorted_list->capacity = cache->num_refs;
+	cache->sorted_list->count = 0;
+
+	ref_list = nwos_malloc_reference_list(list_ref);
+
+	for (i = 0; i < ref_list->common_header.num_refs; i++)
+	{
+	    insert_reference_into_sorted_list(&ref_list->references[i], cache);
+	}
+
+	nwos_free_reference_list(ref_list);
+    }
+
+    return cache->sorted_list;
+}
+
+
+int nwos_find_object_in_sorted_reference_list(Sorted_Reference_List* list, ObjRef* temp_ref)
+{
+    int lwr;
+    int upr;
+    int mid = 0;
+
+    lwr = 1;
+
+    upr = list->count;
+
+    while (lwr <= upr)
+    {
+	mid = (upr + lwr) / 2;
+
+	if (less_than(temp_ref, &list->references[mid - 1]))
+	{
+	    upr = mid - 1;
+	}
+	else if (less_than(&list->references[mid - 1], temp_ref))
+	{
+	    lwr = mid + 1;
+	}
+	else
+	{
+	    mid = mid - 1;  /* adjust to 0 based index */
+	    break;
+	}
+    }
+
+    if (lwr <= upr)
+    {
+	assert(0 <= mid && mid < list->count);
+	assert(!less_than(temp_ref, &list->references[mid]) && !less_than(&list->references[mid], temp_ref));
+
+	while (mid > 0 && !less_than(&list->references[mid-1], temp_ref)) mid--;    /* find the first one if more than one */
+
+	return mid;
+    }
+
+    return -1;   /* not found */
+}
+
+
+#endif
+
+
