@@ -39,8 +39,8 @@
 #      Salt Lake City, UT 84127-0511, USA.
 #
 #   $Author: jsedwards $
-#   $Date: 2010-09-12 14:05:28 -0600 (Sun, 12 Sep 2010) $
-#   $Revision: 4729 $
+#   $Date: 2010-09-19 15:02:28 -0600 (Sun, 19 Sep 2010) $
+#   $Revision: 4758 $
 #
 #   NOTE: Subversion does not support the Log keyword so I have removed the
 #   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -1002,6 +1002,14 @@ if ! grep -q "new revisions:   1" /tmp/test8r.out; then
    exit 1
 fi
 
+if ! $EXEC/src/apps/file/file-verify test2.txt; then
+   echo ""
+   echo "file-verify new revision of file failed"
+   echo ""
+   exit 1
+fi
+
+
 
 echo "-------------------------------------------------"
 echo " Case 11: add revision but path doesn't exist..."
@@ -1023,7 +1031,6 @@ if ! grep -q "path not found:  1" /tmp/test3r.out; then
    exit 1
 fi
 
-
 echo "---------------------------"
 echo " Case 12: revised file..."
 echo "---------------------------"
@@ -1078,6 +1085,60 @@ if ! $EXEC/src/apps/file/file-verify --remove-no-shred test1.txt; then
 fi
 
 
+echo "---------------------------"
+echo " Case 13: update time..."
+echo "---------------------------"
+
+echo "This is test #13" > test13.txt
+
+if ! $EXEC/src/apps/file/file-import test13.txt > /tmp/test13.out; then
+   echo ""
+   echo "file-import test13.txt failed"
+   echo ""
+   exit 1
+fi
+
+if ! $EXEC/src/apps/file/file-verify test13.txt; then
+   echo ""
+   echo "file-verify test13.txt failed"
+   echo ""
+   exit 1
+fi
+
+sleep 3
+
+touch test13.txt
+
+if $EXEC/src/apps/file/file-verify test13.txt; then
+   echo ""
+   echo "file-verify test13.txt with modified time succeeded"
+   echo ""
+   exit 1
+fi
+
+if ! $EXEC/src/apps/file/file-import --update-time test13.txt > /tmp/test13.out; then
+   echo ""
+   echo "file-import test13.txt failed"
+   echo ""
+   exit 1
+fi
+
+if ! grep -q "updated time:    1" /tmp/test13.out; then
+   echo ""
+   echo "did not report 1 updated time file"
+   echo ""
+   exit 1
+fi
+
+if ! $EXEC/src/apps/file/file-verify test13.txt; then
+   echo ""
+   echo "file-verify test13.txt with updated time failed"
+   echo ""
+   exit 1
+fi
+
+
+
 echo ""
 echo "All tests passed!"
 echo ""
