@@ -1,6 +1,6 @@
 #
 #          This file is part of the New World OS and Objectify projects
-#                        Copyright (C) 2009  QRW Software
+#                     Copyright (C) 2009, 2010  QRW Software
 #               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -17,8 +17,11 @@
 #   along with this program, in the file LICENSE.  If not, see 
 #   <http://www.gnu.org/licenses/>.
 #
-#   For the latest information, source code (SVN), releases, bug and feature
-#   request tracking go to:
+#   For the latest information, source code (SVN), releases, and bug tracking
+#   go to:
+#      http://savannah.nongnu.org/projects/objectify
+#
+#   For releases from Alpha_30 and up, bug and feature request tracking go to:
 #      http://sourceforge.net/projects/objectify
 #
 #   For older bug tracking, releases and source code (CVS) prior to the
@@ -36,8 +39,8 @@
 #      Salt Lake City, UT 84127-0511, USA.
 #
 #   $Author: jsedwards $
-#   $Date: 2009-09-20 13:04:12 -0600 (Sun, 20 Sep 2009) $
-#   $Revision: 4372 $
+#   $Date: 2010-01-30 08:57:28 -0700 (Sat, 30 Jan 2010) $
+#   $Revision: 4510 $
 #
 #   NOTE: Subversion does not support the Log keyword so I have removed the
 #   logs that were here when I was using CVS.  Use the "svn log" command to
@@ -104,125 +107,125 @@ cat    x3670016    x3670016 >   x7340032
 echo "---------------"
 echo " prep disk..."
 echo "---------------"
-if ! $EXEC/src/apps/util/prep_archive; then
+if ! $EXEC/src/apps/util/archive-prep; then
    echo ""
-   echo "prep_archive failed"
+   echo "archive-prep failed"
    echo ""
    exit 1
 fi
 
 echo "---------------"
-echo " import_files..."
+echo " file-imports..."
 echo "---------------"
-if ! $EXEC/src/apps/file/import_file --extreme-security x14 x917504 x112; then
+if ! $EXEC/src/apps/file/file-import --extreme-security x14 x917504 x112; then
    echo ""
-   echo "import_file failed"
+   echo "file-import failed"
    echo ""
    exit 1
 fi
 
 echo "---------------"
-echo " import_files..."
+echo " file-imports..."
 echo "---------------"
-if ! $EXEC/src/apps/file/import_file --minimal-security x7*; then
+if ! $EXEC/src/apps/file/file-import --minimal-security x7*; then
    echo ""
-   echo "import_file failed"
+   echo "file-import failed"
    echo ""
    exit 1
 fi
 
 echo "---------------"
-echo " import_files..."
+echo " file-imports..."
 echo "---------------"
-if ! $EXEC/src/apps/file/import_file --low-security x114688; then
+if ! $EXEC/src/apps/file/file-import --low-security x114688; then
    echo ""
-   echo "import_file failed"
+   echo "file-import failed"
    echo ""
    exit 1
 fi
 
 echo "---------------"
-echo " import_files..."
+echo " file-imports..."
 echo "---------------"
-if ! $EXEC/src/apps/file/import_file --medium-security x4*; then
+if ! $EXEC/src/apps/file/file-import --medium-security x4*; then
    echo ""
-   echo "import_file failed"
+   echo "file-import failed"
    echo ""
    exit 1
 fi
 
 echo "---------------"
-echo " import_files..."
+echo " file-imports..."
 echo "---------------"
-if ! $EXEC/src/apps/file/import_file --very-low-security x3*; then
+if ! $EXEC/src/apps/file/file-import --very-low-security x3*; then
    echo ""
-   echo "import_file failed"
+   echo "file-import failed"
    echo ""
    exit 1
 fi
 
 echo "---------------"
-echo " import_files..."
+echo " file-imports..."
 echo "---------------"
-if ! $EXEC/src/apps/file/import_file --high-security x2*; then
+if ! $EXEC/src/apps/file/file-import --high-security x2*; then
    echo ""
-   echo "import_file failed"
+   echo "file-import failed"
    echo ""
    exit 1
 fi
 
 
 echo "---------------"
-echo " import_files..."
+echo " file-imports..."
 echo "---------------"
-if ! $EXEC/src/apps/file/import_file --extreme-security x1792 x14336; then
+if ! $EXEC/src/apps/file/file-import --extreme-security x1792 x14336; then
    echo ""
-   echo "import_file failed"
+   echo "file-import failed"
    echo ""
    exit 1
 fi
 
 echo "---------------"
-echo " import_files..."
+echo " file-imports..."
 echo "---------------"
-if ! $EXEC/src/apps/file/import_file --low-security x5*; then
+if ! $EXEC/src/apps/file/file-import --low-security x5*; then
    echo ""
-   echo "import_file failed"
+   echo "file-import failed"
    echo ""
    exit 1
 fi
 
 echo "---------------"
-echo " import_files..."
+echo " file-imports..."
 echo "---------------"
-if ! $EXEC/src/apps/file/import_file --high-security x896; then
+if ! $EXEC/src/apps/file/file-import --high-security x896; then
    echo ""
-   echo "import_file failed"
+   echo "file-import failed"
    echo ""
    exit 1
 fi
 
 
 echo "---------------"
-echo " import_files..."
+echo " file-imports..."
 echo "---------------"
-if ! $EXEC/src/apps/file/import_file --minimal-security x1835008; then
+if ! $EXEC/src/apps/file/file-import --minimal-security x1835008; then
    echo ""
-   echo "import_file failed"
+   echo "file-import failed"
    echo ""
    exit 1
 fi
 
 echo "-----------------"
-echo " export_files..."
+echo " file-exports..."
 echo "-----------------"
 
 mkdir test
 cd test
 
-if ! $EXEC/src/apps/file/export_file "*"; then
+if ! $EXEC/src/apps/file/file-export "*"; then
    echo ""
-   echo "export_file failed"
+   echo "file-export failed"
    echo ""
    exit 1
 fi
@@ -233,9 +236,9 @@ sleep 2
 echo "---------------------"
 echo " verify and remove..."
 echo "---------------------"
-if ! $EXEC/src/apps/file/verify_file --remove-no-shred *; then
+if ! $EXEC/src/apps/file/file-verify --remove-no-shred *; then
    echo ""
-   echo "verify_file --remove-no-shred succeeded"
+   echo "file-verify --remove-no-shred succeeded"
    echo ""
    exit 1
 fi
@@ -244,9 +247,9 @@ fi
 echo "---------------------"
 echo " compress..."
 echo "---------------------"
-if ! $EXEC/src/apps/util/compress_archive ../compressed; then
+if ! $EXEC/src/apps/util/archive-compress ../compressed; then
    echo ""
-   echo "compress_archive failed"
+   echo "archive-compress failed"
    echo ""
    exit 1
 fi
@@ -255,9 +258,9 @@ fi
 echo "--------------------------"
 echo " prep disk for restore..."
 echo "--------------------------"
-if ! $EXEC/src/apps/util/prep_archive --no-private; then
+if ! $EXEC/src/apps/util/archive-prep --no-private; then
    echo ""
-   echo "prep_archive --no-private failed"
+   echo "archive-prep --no-private failed"
    echo ""
    exit 1
 fi
@@ -266,18 +269,18 @@ fi
 echo "---------------------"
 echo " expand..."
 echo "---------------------"
-if ! $EXEC/src/apps/util/restore_archive ../compressed; then
+if ! $EXEC/src/apps/util/archive-restore ../compressed; then
    echo ""
-   echo "restore_archive failed"
+   echo "archive-restore failed"
    echo ""
    exit 1
 fi
 
 
 
-if ! $EXEC/src/apps/file/export_file "*"; then
+if ! $EXEC/src/apps/file/file-export "*"; then
    echo ""
-   echo "export_file failed"
+   echo "file-export failed"
    echo ""
    exit 1
 fi
@@ -288,9 +291,9 @@ for file in ../*.[ch]; do diff -s $file .; done
 echo "---------------------"
 echo " verify and remove..."
 echo "---------------------"
-if ! $EXEC/src/apps/file/verify_file --remove-no-shred *; then
+if ! $EXEC/src/apps/file/file-verify --remove-no-shred *; then
    echo ""
-   echo "verify_file --remove-no-shred failed"
+   echo "file-verify --remove-no-shred failed"
    echo ""
    exit 1
 fi
